// 
// Copyright (c) Microsoft.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Common;
using Microsoft.WindowsAzure.Common.Internals;
using Microsoft.WindowsAzure.Management.Monitoring.Alerts;
using Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models
{
    public enum ConditionOperator
    {
        GreaterThan = 0,

        GreaterThanOrEqual = 1,

        LessThan = 2,

        LessThanOrEqual = 3,
    }

    /// <summary>
    /// An alert incident indicates the activation status of an alert rule.
    /// </summary>
    public partial class Incident
    {
        private DateTime _activatedTime;

        /// <summary>
        /// The time at which the incident got activated.
        /// </summary>
        public DateTime ActivatedTime
        {
            get { return this._activatedTime; }
            set { this._activatedTime = value; }
        }

        private string _id;

        /// <summary>
        /// Incident identifier.
        /// </summary>
        public string Id
        {
            get { return this._id; }
            set { this._id = value; }
        }

        private bool _isActive;

        /// <summary>
        /// A boolean to indicate whether the incident is active or resolved.
        /// </summary>
        public bool IsActive
        {
            get { return this._isActive; }
            set { this._isActive = value; }
        }

        private System.DateTime? _resolvedTime;

        /// <summary>
        /// The time at which the incident got resolved. If null, it means the
        /// incident is still active.
        /// </summary>
        public System.DateTime? ResolvedTime
        {
            get { return this._resolvedTime; }
            set { this._resolvedTime = value; }
        }

        private string _ruleId;

        /// <summary>
        /// Rule identifier that is associated with the incident.
        /// </summary>
        public string RuleId
        {
            get { return this._ruleId; }
            set { this._ruleId = value; }
        }

        /// <summary>
        /// Initializes a new instance of the Incident class.
        /// </summary>
        public Incident()
        {
        }
    }

    /// <summary>
    /// Alert incidents collection.
    /// </summary>
    public partial class IncidentCollection
    {
        private IList<Incident> _value;

        /// <summary>
        /// Incident collection.
        /// </summary>
        public IList<Incident> Value
        {
            get { return this._value; }
            set { this._value = value; }
        }

        /// <summary>
        /// Initializes a new instance of the IncidentCollection class.
        /// </summary>
        public IncidentCollection()
        {
            this._value = new List<Incident>();
        }
    }

    /// <summary>
    /// The Get Incident operation response.
    /// </summary>
    public partial class IncidentGetResponse : OperationResponse
    {
        private Incident _incident;

        /// <summary>
        /// The retrieved incident.
        /// </summary>
        public Incident Incident
        {
            get { return this._incident; }
            set { this._incident = value; }
        }

        /// <summary>
        /// Initializes a new instance of the IncidentGetResponse class.
        /// </summary>
        public IncidentGetResponse()
        {
        }
    }

    /// <summary>
    /// The List incidents operation response.
    /// </summary>
    public partial class IncidentListResponse : OperationResponse
    {
        private IncidentCollection _incidentCollection;

        /// <summary>
        /// The retrieved incidents.
        /// </summary>
        public IncidentCollection IncidentCollection
        {
            get { return this._incidentCollection; }
            set { this._incidentCollection = value; }
        }

        /// <summary>
        /// Initializes a new instance of the IncidentListResponse class.
        /// </summary>
        public IncidentListResponse()
        {
        }
    }

    /// <summary>
    /// An alert rule.
    /// </summary>
    public partial class Rule
    {
        private IList<RuleAction> _actions;

        /// <summary>
        /// Rule actions.
        /// </summary>
        public IList<RuleAction> Actions
        {
            get { return this._actions; }
            set { this._actions = value; }
        }

        private RuleCondition _condition;

        /// <summary>
        /// Rule condition.
        /// </summary>
        public RuleCondition Condition
        {
            get { return this._condition; }
            set { this._condition = value; }
        }

        private string _description;

        /// <summary>
        /// Rule description.
        /// </summary>
        public string Description
        {
            get { return this._description; }
            set { this._description = value; }
        }

        private string _id;

        /// <summary>
        /// Rule identifier.
        /// </summary>
        public string Id
        {
            get { return this._id; }
            set { this._id = value; }
        }

        private bool _isEnabled;

        /// <summary>
        /// A flag that determines whether the rule is enabled or disabled.
        /// </summary>
        public bool IsEnabled
        {
            get { return this._isEnabled; }
            set { this._isEnabled = value; }
        }

        private DateTime _lastUpdatedTime;

        /// <summary>
        /// Rule last updated time.
        /// </summary>
        public DateTime LastUpdatedTime
        {
            get { return this._lastUpdatedTime; }
            set { this._lastUpdatedTime = value; }
        }

        private string _name;

        /// <summary>
        /// Rule name.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }

        /// <summary>
        /// Initializes a new instance of the Rule class.
        /// </summary>
        public Rule()
        {
            this._actions = new List<RuleAction>();
        }
    }

    /// <summary>
    /// A rule action.
    /// </summary>
    public abstract partial class RuleAction
    {
        /// <summary>
        /// Initializes a new instance of the RuleAction class.
        /// </summary>
        public RuleAction()
        {
        }
    }

    /// <summary>
    /// Alert rules collection.
    /// </summary>
    public partial class RuleCollection
    {
        private IList<Rule> _value;

        /// <summary>
        /// Rule collection.
        /// </summary>
        public IList<Rule> Value
        {
            get { return this._value; }
            set { this._value = value; }
        }

        /// <summary>
        /// Initializes a new instance of the RuleCollection class.
        /// </summary>
        public RuleCollection()
        {
            this._value = new List<Rule>();
        }
    }

    /// <summary>
    /// A rule condition.
    /// </summary>
    public abstract partial class RuleCondition
    {
        /// <summary>
        /// Initializes a new instance of the RuleCondition class.
        /// </summary>
        public RuleCondition()
        {
        }
    }

    /// <summary>
    /// Parameters supplied to the Create or Update Rule operation.
    /// </summary>
    public partial class RuleCreateOrUpdateParameters
    {
        private Rule _rule;

        /// <summary>
        /// The rule to create or update.
        /// </summary>
        public Rule Rule
        {
            get { return this._rule; }
            set { this._rule = value; }
        }

        /// <summary>
        /// Initializes a new instance of the RuleCreateOrUpdateParameters
        /// class.
        /// </summary>
        public RuleCreateOrUpdateParameters()
        {
        }
    }

    /// <summary>
    /// A rule data source.
    /// </summary>
    public abstract partial class RuleDataSource
    {
        /// <summary>
        /// Initializes a new instance of the RuleDataSource class.
        /// </summary>
        public RuleDataSource()
        {
        }
    }

    /// <summary>
    /// Specifies the action to send email when the rule condition is evaluated
    /// </summary>
    public partial class RuleEmailAction : RuleAction
    {
        private IList<string> _customEmails;

        /// <summary>
        /// The email address of an adminstrative user.
        /// </summary>
        public IList<string> CustomEmails
        {
            get { return this._customEmails; }
            set { this._customEmails = value; }
        }

        private bool _sendToServiceOwners;

        /// <summary>
        /// This indicates if email is sent to sevice adminstrator and
        /// co-administrators.
        /// </summary>
        public bool SendToServiceOwners
        {
            get { return this._sendToServiceOwners; }
            set { this._sendToServiceOwners = value; }
        }

        /// <summary>
        /// Initializes a new instance of the RuleEmailAction class.
        /// </summary>
        public RuleEmailAction()
        {
            this._customEmails = new List<string>();
        }
    }

    /// <summary>
    /// The Get Rule operation response.
    /// </summary>
    public partial class RuleGetResponse : OperationResponse
    {
        private Rule _rule;

        /// <summary>
        /// The retrieved rule.
        /// </summary>
        public Rule Rule
        {
            get { return this._rule; }
            set { this._rule = value; }
        }

        /// <summary>
        /// Initializes a new instance of the RuleGetResponse class.
        /// </summary>
        public RuleGetResponse()
        {
        }
    }

    /// <summary>
    /// The List Rules operation response.
    /// </summary>
    public partial class RuleListResponse : OperationResponse
    {
        private RuleCollection _ruleCollection;

        /// <summary>
        /// The retrieved rules.
        /// </summary>
        public RuleCollection RuleCollection
        {
            get { return this._ruleCollection; }
            set { this._ruleCollection = value; }
        }

        /// <summary>
        /// Initializes a new instance of the RuleListResponse class.
        /// </summary>
        public RuleListResponse()
        {
        }
    }

    /// <summary>
    /// A rule metric data source.
    /// </summary>
    public partial class RuleMetricDataSource : RuleDataSource
    {
        private string _metricName;

        /// <summary>
        /// Metric name.
        /// </summary>
        public string MetricName
        {
            get { return this._metricName; }
            set { this._metricName = value; }
        }

        private string _metricNamespace;

        /// <summary>
        /// Metric namespace. When creating a rule on endpoint monitoring
        /// metrics, WindowsAzure.Availability namespace is required.
        /// </summary>
        public string MetricNamespace
        {
            get { return this._metricNamespace; }
            set { this._metricNamespace = value; }
        }

        private string _resourceId;

        /// <summary>
        /// Metric resource id.
        /// </summary>
        public string ResourceId
        {
            get { return this._resourceId; }
            set { this._resourceId = value; }
        }

        /// <summary>
        /// Initializes a new instance of the RuleMetricDataSource class.
        /// </summary>
        public RuleMetricDataSource()
        {
        }
    }

    /// <summary>
    /// A threshold rule condition.
    /// </summary>
    public partial class ThresholdRuleCondition : RuleCondition
    {
        private RuleDataSource _dataSource;

        /// <summary>
        /// Condition data source.
        /// </summary>
        public RuleDataSource DataSource
        {
            get { return this._dataSource; }
            set { this._dataSource = value; }
        }

        private ConditionOperator _operator;

        /// <summary>
        /// Condition operator.
        /// </summary>
        public ConditionOperator Operator
        {
            get { return this._operator; }
            set { this._operator = value; }
        }

        private double _threshold;

        /// <summary>
        /// Condition threshold.
        /// </summary>
        public double Threshold
        {
            get { return this._threshold; }
            set { this._threshold = value; }
        }

        private TimeSpan _windowSize;

        /// <summary>
        /// The time period over which the alert rule is evaluated. Condition
        /// window size depends on the metric.
        /// </summary>
        public TimeSpan WindowSize
        {
            get { return this._windowSize; }
            set { this._windowSize = value; }
        }

        /// <summary>
        /// Initializes a new instance of the ThresholdRuleCondition class.
        /// </summary>
        public ThresholdRuleCondition()
        {
        }
    }
}

namespace Microsoft.WindowsAzure.Management.Monitoring.Alerts
{
    public partial interface IAlertsClient
    {
        /// <summary>
        /// Optional base uri parameter for Azure REST.
        /// </summary>
        Uri BaseUri
        {
            get;
        }

        /// <summary>
        /// When you create a Windows Azure subscription, it is uniquely
        /// identified by a subscription ID. The subscription ID forms part of
        /// the URI for every call that you make to the Service Management
        /// API.  The Windows Azure Service ManagementAPI use mutual
        /// authentication of management certificates over SSL to ensure that
        /// a request made to the service is secure.  No anonymous requests
        /// are allowed.
        /// </summary>
        SubscriptionCloudCredentials Credentials
        {
            get;
        }

        /// <summary>
        /// Operations for managing the alert incidents.
        /// </summary>
        IIncidentOperations Incidents
        {
            get;
        }

        /// <summary>
        /// Operations for managing the alert rules.
        /// </summary>
        IRuleOperations Rules
        {
            get;
        }
    }

    public static partial class AlertsClientExtensions
    {
    }

    public partial class AlertsClient : ServiceClient<AlertsClient>, IAlertsClient
    {
        private Uri _baseUri;

        /// <summary>
        /// Optional base uri parameter for Azure REST.
        /// </summary>
        public Uri BaseUri
        {
            get { return this._baseUri; }
        }

        private SubscriptionCloudCredentials _credentials;

        /// <summary>
        /// When you create a Windows Azure subscription, it is uniquely
        /// identified by a subscription ID. The subscription ID forms part of
        /// the URI for every call that you make to the Service Management
        /// API.  The Windows Azure Service ManagementAPI use mutual
        /// authentication of management certificates over SSL to ensure that
        /// a request made to the service is secure.  No anonymous requests
        /// are allowed.
        /// </summary>
        public SubscriptionCloudCredentials Credentials
        {
            get { return this._credentials; }
        }

        private IIncidentOperations _incidents;

        /// <summary>
        /// Operations for managing the alert incidents.
        /// </summary>
        public virtual IIncidentOperations Incidents
        {
            get { return this._incidents; }
        }

        private IRuleOperations _rules;

        /// <summary>
        /// Operations for managing the alert rules.
        /// </summary>
        public virtual IRuleOperations Rules
        {
            get { return this._rules; }
        }

        /// <summary>
        /// Initializes a new instance of the AlertsClient class.
        /// </summary>
        private AlertsClient()
            : base()
        {
            this._incidents = new IncidentOperations(this);
            this._rules = new RuleOperations(this);
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }

        /// <summary>
        /// Initializes a new instance of the AlertsClient class.
        /// </summary>
        /// <param name='credentials'>
        /// When you create a Windows Azure subscription, it is uniquely
        /// identified by a subscription ID. The subscription ID forms part of
        /// the URI for every call that you make to the Service Management
        /// API.  The Windows Azure Service ManagementAPI use mutual
        /// authentication of management certificates over SSL to ensure that
        /// a request made to the service is secure.  No anonymous requests
        /// are allowed.
        /// </param>
        /// <param name='baseUri'>
        /// Optional base uri parameter for Azure REST.
        /// </param>
        public AlertsClient(SubscriptionCloudCredentials credentials, Uri baseUri)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._credentials = credentials;
            this._baseUri = baseUri;

            this.Credentials.InitializeServiceClient(this);
        }

        /// <summary>
        /// Initializes a new instance of the AlertsClient class.
        /// </summary>
        /// <param name='credentials'>
        /// When you create a Windows Azure subscription, it is uniquely
        /// identified by a subscription ID. The subscription ID forms part of
        /// the URI for every call that you make to the Service Management
        /// API.  The Windows Azure Service ManagementAPI use mutual
        /// authentication of management certificates over SSL to ensure that
        /// a request made to the service is secure.  No anonymous requests
        /// are allowed.
        /// </param>
        public AlertsClient(SubscriptionCloudCredentials credentials)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._credentials = credentials;
            this._baseUri = new Uri("https://management.core.windows.net");

            this.Credentials.InitializeServiceClient(this);
        }
    }

    /// <summary>
    /// Operations for managing the alert incidents.
    /// </summary>
    public partial interface IIncidentOperations
    {
        /// <param name='incidentId'>
        /// The id of the incident to retrieve.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Incident operation response.
        /// </returns>
        Task<IncidentGetResponse> GetAsync(string incidentId, CancellationToken cancellationToken);

        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        Task<IncidentListResponse> ListActiveForSubscriptionAsync(CancellationToken cancellationToken);

        /// <param name='ruleId'>
        /// The rule id.
        /// </param>
        /// <param name='isActive'>
        /// A boolean to retrieve only active or resolved incidents.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        Task<IncidentListResponse> ListForRuleAsync(string ruleId, bool isActive, CancellationToken cancellationToken);
    }

    /// <summary>
    /// Operations for managing the alert incidents.
    /// </summary>
    public static partial class IncidentOperationsExtensions
    {
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IIncidentOperations.
        /// </param>
        /// <param name='incidentId'>
        /// The id of the incident to retrieve.
        /// </param>
        /// <returns>
        /// The Get Incident operation response.
        /// </returns>
        public static IncidentGetResponse Get(this IIncidentOperations operations, string incidentId)
        {
            try
            {
                return operations.GetAsync(incidentId).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IIncidentOperations.
        /// </param>
        /// <param name='incidentId'>
        /// The id of the incident to retrieve.
        /// </param>
        /// <returns>
        /// The Get Incident operation response.
        /// </returns>
        public static Task<IncidentGetResponse> GetAsync(this IIncidentOperations operations, string incidentId)
        {
            return operations.GetAsync(incidentId, CancellationToken.None);
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IIncidentOperations.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        public static IncidentListResponse ListActiveForSubscription(this IIncidentOperations operations)
        {
            try
            {
                return operations.ListActiveForSubscriptionAsync().Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IIncidentOperations.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        public static Task<IncidentListResponse> ListActiveForSubscriptionAsync(this IIncidentOperations operations)
        {
            return operations.ListActiveForSubscriptionAsync(CancellationToken.None);
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IIncidentOperations.
        /// </param>
        /// <param name='ruleId'>
        /// The rule id.
        /// </param>
        /// <param name='isActive'>
        /// A boolean to retrieve only active or resolved incidents.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        public static IncidentListResponse ListForRule(this IIncidentOperations operations, string ruleId, bool isActive)
        {
            try
            {
                return operations.ListForRuleAsync(ruleId, isActive).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IIncidentOperations.
        /// </param>
        /// <param name='ruleId'>
        /// The rule id.
        /// </param>
        /// <param name='isActive'>
        /// A boolean to retrieve only active or resolved incidents.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        public static Task<IncidentListResponse> ListForRuleAsync(this IIncidentOperations operations, string ruleId, bool isActive)
        {
            return operations.ListForRuleAsync(ruleId, isActive, CancellationToken.None);
        }
    }

    /// <summary>
    /// Operations for managing the alert incidents.
    /// </summary>
    internal partial class IncidentOperations : IServiceOperations<AlertsClient>, IIncidentOperations
    {
        /// <summary>
        /// Initializes a new instance of the IncidentOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal IncidentOperations(AlertsClient client)
        {
            this._client = client;
        }

        private AlertsClient _client;

        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.AlertsClient.
        /// </summary>
        public AlertsClient Client
        {
            get { return this._client; }
        }

        /// <param name='incidentId'>
        /// The id of the incident to retrieve.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Incident operation response.
        /// </returns>
        public async Task<IncidentGetResponse> GetAsync(string incidentId, CancellationToken cancellationToken)
        {
            // Validate
            if (incidentId == null)
            {
                throw new ArgumentNullException("incidentId");
            }

            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("incidentId", incidentId);
                Tracing.Enter(invocationId, this, "GetAsync", tracingParameters);
            }

            // Construct URL
            string url = this.Client.BaseUri + "/" + this.Client.Credentials.SubscriptionId + "/services/monitoring/alertincidents/" + incidentId;

            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);

                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");

                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);

                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }

                    // Create Result
                    IncidentGetResponse result = new IncidentGetResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }

                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);

                    if (responseDoc != null)
                    {
                        Incident incidentInstance = new Incident();
                        result.Incident = incidentInstance;

                        JToken idValue = responseDoc["Id"];
                        if (idValue != null)
                        {
                            string idInstance = (string)idValue;
                            incidentInstance.Id = idInstance;
                        }

                        JToken ruleIdValue = responseDoc["RuleId"];
                        if (ruleIdValue != null)
                        {
                            string ruleIdInstance = (string)ruleIdValue;
                            incidentInstance.RuleId = ruleIdInstance;
                        }

                        JToken isActiveValue = responseDoc["IsActive"];
                        if (isActiveValue != null)
                        {
                            bool isActiveInstance = (bool)isActiveValue;
                            incidentInstance.IsActive = isActiveInstance;
                        }

                        JToken activatedTimeValue = responseDoc["ActivatedTime"];
                        if (activatedTimeValue != null)
                        {
                            DateTime activatedTimeInstance = (DateTime)activatedTimeValue;
                            incidentInstance.ActivatedTime = activatedTimeInstance;
                        }

                        JToken resolvedTimeValue = responseDoc["ResolvedTime"];
                        if (resolvedTimeValue != null)
                        {
                            DateTime resolvedTimeInstance = (DateTime)resolvedTimeValue;
                            incidentInstance.ResolvedTime = resolvedTimeInstance;
                        }
                    }

                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }

        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        public async Task<IncidentListResponse> ListActiveForSubscriptionAsync(CancellationToken cancellationToken)
        {
            // Validate

            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                Tracing.Enter(invocationId, this, "ListActiveForSubscriptionAsync", tracingParameters);
            }

            // Construct URL
            string url = this.Client.BaseUri + "/" + this.Client.Credentials.SubscriptionId + "/services/monitoring/alertincidents?$filter=IsActive eq true";

            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);

                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");

                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);

                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }

                    // Create Result
                    IncidentListResponse result = new IncidentListResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }

                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);

                    if (responseDoc != null)
                    {
                        IncidentCollection incidentCollectionInstance = new IncidentCollection();
                        result.IncidentCollection = incidentCollectionInstance;

                        JArray valueArray = (JArray)responseDoc["Value"];
                        if (valueArray != null)
                        {
                            foreach (JToken valueValue in valueArray)
                            {
                                Incident incidentInstance = new Incident();
                                incidentCollectionInstance.Value.Add(incidentInstance);

                                JToken idValue = valueValue["Id"];
                                if (idValue != null)
                                {
                                    string idInstance = (string)idValue;
                                    incidentInstance.Id = idInstance;
                                }

                                JToken ruleIdValue = valueValue["RuleId"];
                                if (ruleIdValue != null)
                                {
                                    string ruleIdInstance = (string)ruleIdValue;
                                    incidentInstance.RuleId = ruleIdInstance;
                                }

                                JToken isActiveValue = valueValue["IsActive"];
                                if (isActiveValue != null)
                                {
                                    bool isActiveInstance = (bool)isActiveValue;
                                    incidentInstance.IsActive = isActiveInstance;
                                }

                                JToken activatedTimeValue = valueValue["ActivatedTime"];
                                if (activatedTimeValue != null)
                                {
                                    DateTime activatedTimeInstance = (DateTime)activatedTimeValue;
                                    incidentInstance.ActivatedTime = activatedTimeInstance;
                                }

                                JToken resolvedTimeValue = valueValue["ResolvedTime"];
                                if (resolvedTimeValue != null)
                                {
                                    DateTime resolvedTimeInstance = (DateTime)resolvedTimeValue;
                                    incidentInstance.ResolvedTime = resolvedTimeInstance;
                                }
                            }
                        }
                    }

                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }

        /// <param name='ruleId'>
        /// The rule id.
        /// </param>
        /// <param name='isActive'>
        /// A boolean to retrieve only active or resolved incidents.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List incidents operation response.
        /// </returns>
        public async Task<IncidentListResponse> ListForRuleAsync(string ruleId, bool isActive, CancellationToken cancellationToken)
        {
            // Validate
            if (ruleId == null)
            {
                throw new ArgumentNullException("ruleId");
            }

            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("ruleId", ruleId);
                tracingParameters.Add("isActive", isActive);
                Tracing.Enter(invocationId, this, "ListForRuleAsync", tracingParameters);
            }

            // Construct URL
            string url = this.Client.BaseUri + "/" + this.Client.Credentials.SubscriptionId + "/services/monitoring/alertrules/" + ruleId + "/alertincidents?$filter=IsActive eq " + isActive;

            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);

                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");

                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);

                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }

                    // Create Result
                    IncidentListResponse result = new IncidentListResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }

                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);

                    if (responseDoc != null)
                    {
                        IncidentCollection incidentCollectionInstance = new IncidentCollection();
                        result.IncidentCollection = incidentCollectionInstance;

                        JArray valueArray = (JArray)responseDoc["Value"];
                        if (valueArray != null)
                        {
                            foreach (JToken valueValue in valueArray)
                            {
                                Incident incidentInstance = new Incident();
                                incidentCollectionInstance.Value.Add(incidentInstance);

                                JToken idValue = valueValue["Id"];
                                if (idValue != null)
                                {
                                    string idInstance = (string)idValue;
                                    incidentInstance.Id = idInstance;
                                }

                                JToken ruleIdValue = valueValue["RuleId"];
                                if (ruleIdValue != null)
                                {
                                    string ruleIdInstance = (string)ruleIdValue;
                                    incidentInstance.RuleId = ruleIdInstance;
                                }

                                JToken isActiveValue = valueValue["IsActive"];
                                if (isActiveValue != null)
                                {
                                    bool isActiveInstance = (bool)isActiveValue;
                                    incidentInstance.IsActive = isActiveInstance;
                                }

                                JToken activatedTimeValue = valueValue["ActivatedTime"];
                                if (activatedTimeValue != null)
                                {
                                    DateTime activatedTimeInstance = (DateTime)activatedTimeValue;
                                    incidentInstance.ActivatedTime = activatedTimeInstance;
                                }

                                JToken resolvedTimeValue = valueValue["ResolvedTime"];
                                if (resolvedTimeValue != null)
                                {
                                    DateTime resolvedTimeInstance = (DateTime)resolvedTimeValue;
                                    incidentInstance.ResolvedTime = resolvedTimeInstance;
                                }
                            }
                        }
                    }

                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }

    /// <summary>
    /// Operations for managing the alert rules.
    /// </summary>
    public partial interface IRuleOperations
    {
        /// <param name='parameters'>
        /// The rule to create or update.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<OperationResponse> CreateOrUpdateAsync(RuleCreateOrUpdateParameters parameters, CancellationToken cancellationToken);

        /// <param name='ruleId'>
        /// The id of the rule to delete.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<OperationResponse> DeleteAsync(string ruleId, CancellationToken cancellationToken);

        /// <param name='ruleId'>
        /// The id of the rule to retrieve.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Rule operation response.
        /// </returns>
        Task<RuleGetResponse> GetAsync(string ruleId, CancellationToken cancellationToken);

        /// <summary>
        /// List the alert rules within a subscription
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List Rules operation response.
        /// </returns>
        Task<RuleListResponse> ListAsync(CancellationToken cancellationToken);
    }

    /// <summary>
    /// Operations for managing the alert rules.
    /// </summary>
    public static partial class RuleOperationsExtensions
    {
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <param name='parameters'>
        /// The rule to create or update.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse CreateOrUpdate(this IRuleOperations operations, RuleCreateOrUpdateParameters parameters)
        {
            try
            {
                return operations.CreateOrUpdateAsync(parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <param name='parameters'>
        /// The rule to create or update.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> CreateOrUpdateAsync(this IRuleOperations operations, RuleCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(parameters, CancellationToken.None);
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <param name='ruleId'>
        /// The id of the rule to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse Delete(this IRuleOperations operations, string ruleId)
        {
            try
            {
                return operations.DeleteAsync(ruleId).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <param name='ruleId'>
        /// The id of the rule to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> DeleteAsync(this IRuleOperations operations, string ruleId)
        {
            return operations.DeleteAsync(ruleId, CancellationToken.None);
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <param name='ruleId'>
        /// The id of the rule to retrieve.
        /// </param>
        /// <returns>
        /// The Get Rule operation response.
        /// </returns>
        public static RuleGetResponse Get(this IRuleOperations operations, string ruleId)
        {
            try
            {
                return operations.GetAsync(ruleId).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }

        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <param name='ruleId'>
        /// The id of the rule to retrieve.
        /// </param>
        /// <returns>
        /// The Get Rule operation response.
        /// </returns>
        public static Task<RuleGetResponse> GetAsync(this IRuleOperations operations, string ruleId)
        {
            return operations.GetAsync(ruleId, CancellationToken.None);
        }

        /// <summary>
        /// List the alert rules within a subscription
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <returns>
        /// The List Rules operation response.
        /// </returns>
        public static RuleListResponse List(this IRuleOperations operations)
        {
            try
            {
                return operations.ListAsync().Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }

        /// <summary>
        /// List the alert rules within a subscription
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.IRuleOperations.
        /// </param>
        /// <returns>
        /// The List Rules operation response.
        /// </returns>
        public static Task<RuleListResponse> ListAsync(this IRuleOperations operations)
        {
            return operations.ListAsync(CancellationToken.None);
        }
    }

    /// <summary>
    /// Operations for managing the alert rules.
    /// </summary>
    internal partial class RuleOperations : IServiceOperations<AlertsClient>, IRuleOperations
    {
        /// <summary>
        /// Initializes a new instance of the RuleOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal RuleOperations(AlertsClient client)
        {
            this._client = client;
        }

        private AlertsClient _client;

        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Alerts.AlertsClient.
        /// </summary>
        public AlertsClient Client
        {
            get { return this._client; }
        }

        /// <param name='parameters'>
        /// The rule to create or update.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<OperationResponse> CreateOrUpdateAsync(RuleCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }

            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }

            // Construct URL
            string url = this.Client.BaseUri + "/" + this.Client.Credentials.SubscriptionId + "/services/monitoring/alertrules/" + parameters.Rule.Id;

            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);

                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");

                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);

                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;

                if (parameters.Rule != null)
                {
                    JObject ruleValue = new JObject();
                    requestDoc = ruleValue;

                    if (parameters.Rule.Id != null)
                    {
                        JToken idValue = parameters.Rule.Id;
                        ruleValue["Id"] = idValue;
                    }

                    if (parameters.Rule.Name != null)
                    {
                        JToken nameValue = parameters.Rule.Name;
                        ruleValue["Name"] = nameValue;
                    }

                    if (parameters.Rule.Description != null)
                    {
                        JToken descriptionValue = parameters.Rule.Description;
                        ruleValue["Description"] = descriptionValue;
                    }

                    JToken isEnabledValue = parameters.Rule.IsEnabled;
                    ruleValue["IsEnabled"] = isEnabledValue;

                    if (parameters.Rule.Condition != null)
                    {
                        JObject conditionValue = new JObject();
                        ruleValue["Condition"] = conditionValue;
                        conditionValue["odata.type"] = parameters.Rule.Condition.GetType().FullName;
                        if (parameters.Rule.Condition is ThresholdRuleCondition)
                        {
                            ThresholdRuleCondition derived = (ThresholdRuleCondition)parameters.Rule.Condition;

                            if (derived.DataSource != null)
                            {
                                JObject dataSourceValue = new JObject();
                                conditionValue["DataSource"] = dataSourceValue;
                                dataSourceValue["odata.type"] = derived.DataSource.GetType().FullName;
                                if (derived.DataSource is RuleMetricDataSource)
                                {
                                    RuleMetricDataSource derived2 = (RuleMetricDataSource)derived.DataSource;

                                    if (derived2.ResourceId != null)
                                    {
                                        JToken resourceIdValue = derived2.ResourceId;
                                        dataSourceValue["ResourceId"] = resourceIdValue;
                                    }

                                    if (derived2.MetricNamespace != null)
                                    {
                                        JToken metricNamespaceValue = derived2.MetricNamespace;
                                        dataSourceValue["MetricNamespace"] = metricNamespaceValue;
                                    }

                                    if (derived2.MetricName != null)
                                    {
                                        JToken metricNameValue = derived2.MetricName;
                                        dataSourceValue["MetricName"] = metricNameValue;
                                    }
                                }
                            }

                            JToken operatorValue = derived.Operator.ToString();
                            conditionValue["Operator"] = operatorValue;

                            JToken thresholdValue = derived.Threshold;
                            conditionValue["Threshold"] = thresholdValue;

                            JToken windowSizeValue = TypeConversion.To8601String(derived.WindowSize);
                            conditionValue["WindowSize"] = windowSizeValue;
                        }
                    }

                    if (parameters.Rule.Actions != null)
                    {
                        JArray actionsArray = new JArray();
                        foreach (RuleAction actionsItem in parameters.Rule.Actions)
                        {
                            JObject ruleActionValue = new JObject();
                            actionsArray.Add(ruleActionValue);
                            ruleActionValue["odata.type"] = actionsItem.GetType().FullName;
                            if (actionsItem is RuleEmailAction)
                            {
                                RuleEmailAction derived3 = (RuleEmailAction)actionsItem;

                                JToken sendToServiceOwnersValue = derived3.SendToServiceOwners;
                                ruleActionValue["SendToServiceOwners"] = sendToServiceOwnersValue;

                                if (derived3.CustomEmails != null)
                                {
                                    JArray customEmailsArray = new JArray();
                                    foreach (string customEmailsItem in derived3.CustomEmails)
                                    {
                                        customEmailsArray.Add(customEmailsItem);
                                    }
                                    ruleActionValue["CustomEmails"] = customEmailsArray;
                                }
                            }
                        }
                        ruleValue["Actions"] = actionsArray;
                    }

                    JToken lastUpdatedTimeValue = parameters.Rule.LastUpdatedTime;
                    ruleValue["LastUpdatedTime"] = lastUpdatedTimeValue;
                }

                requestContent = requestDoc.ToString(Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/json");

                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }

                    // Create Result
                    OperationResponse result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }

                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }

        /// <param name='ruleId'>
        /// The id of the rule to delete.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<OperationResponse> DeleteAsync(string ruleId, CancellationToken cancellationToken)
        {
            // Validate
            if (ruleId == null)
            {
                throw new ArgumentNullException("ruleId");
            }

            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("ruleId", ruleId);
                Tracing.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }

            // Construct URL
            string url = this.Client.BaseUri + "/" + this.Client.Credentials.SubscriptionId + "/services/monitoring/alertrules/" + ruleId;

            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);

                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");

                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);

                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }

                    // Create Result
                    OperationResponse result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }

                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }

        /// <param name='ruleId'>
        /// The id of the rule to retrieve.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Rule operation response.
        /// </returns>
        public async Task<RuleGetResponse> GetAsync(string ruleId, CancellationToken cancellationToken)
        {
            // Validate
            if (ruleId == null)
            {
                throw new ArgumentNullException("ruleId");
            }

            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("ruleId", ruleId);
                Tracing.Enter(invocationId, this, "GetAsync", tracingParameters);
            }

            // Construct URL
            string url = this.Client.BaseUri + "/" + this.Client.Credentials.SubscriptionId + "/services/monitoring/alertrules/" + ruleId;

            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);

                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");

                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);

                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }

                    // Create Result
                    RuleGetResponse result = new RuleGetResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }

                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);

                    if (responseDoc != null)
                    {
                        Rule ruleInstance = new Rule();
                        result.Rule = ruleInstance;

                        JToken idValue = responseDoc["Id"];
                        if (idValue != null)
                        {
                            string idInstance = (string)idValue;
                            ruleInstance.Id = idInstance;
                        }

                        JToken nameValue = responseDoc["Name"];
                        if (nameValue != null)
                        {
                            string nameInstance = (string)nameValue;
                            ruleInstance.Name = nameInstance;
                        }

                        JToken descriptionValue = responseDoc["Description"];
                        if (descriptionValue != null)
                        {
                            string descriptionInstance = (string)descriptionValue;
                            ruleInstance.Description = descriptionInstance;
                        }

                        JToken isEnabledValue = responseDoc["IsEnabled"];
                        if (isEnabledValue != null)
                        {
                            bool isEnabledInstance = (bool)isEnabledValue;
                            ruleInstance.IsEnabled = isEnabledInstance;
                        }

                        JToken conditionValue = responseDoc["Condition"];
                        if (conditionValue != null)
                        {
                            string typeName = (string)conditionValue["odata.type"];
                            if (typeName == "Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models.ThresholdRuleCondition")
                            {
                                ThresholdRuleCondition thresholdRuleConditionInstance = new ThresholdRuleCondition();

                                JToken dataSourceValue = conditionValue["DataSource"];
                                if (dataSourceValue != null)
                                {
                                    string typeName2 = (string)dataSourceValue["odata.type"];
                                    if (typeName2 == "Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models.RuleMetricDataSource")
                                    {
                                        RuleMetricDataSource ruleMetricDataSourceInstance = new RuleMetricDataSource();

                                        JToken resourceIdValue = dataSourceValue["ResourceId"];
                                        if (resourceIdValue != null)
                                        {
                                            string resourceIdInstance = (string)resourceIdValue;
                                            ruleMetricDataSourceInstance.ResourceId = resourceIdInstance;
                                        }

                                        JToken metricNamespaceValue = dataSourceValue["MetricNamespace"];
                                        if (metricNamespaceValue != null)
                                        {
                                            string metricNamespaceInstance = (string)metricNamespaceValue;
                                            ruleMetricDataSourceInstance.MetricNamespace = metricNamespaceInstance;
                                        }

                                        JToken metricNameValue = dataSourceValue["MetricName"];
                                        if (metricNameValue != null)
                                        {
                                            string metricNameInstance = (string)metricNameValue;
                                            ruleMetricDataSourceInstance.MetricName = metricNameInstance;
                                        }
                                        thresholdRuleConditionInstance.DataSource = ruleMetricDataSourceInstance;
                                    }
                                }

                                JToken operatorValue = conditionValue["Operator"];
                                if (operatorValue != null)
                                {
                                    ConditionOperator operatorInstance = (ConditionOperator)Enum.Parse(typeof(ConditionOperator), (string)operatorValue, false);
                                    thresholdRuleConditionInstance.Operator = operatorInstance;
                                }

                                JToken thresholdValue = conditionValue["Threshold"];
                                if (thresholdValue != null)
                                {
                                    double thresholdInstance = (double)thresholdValue;
                                    thresholdRuleConditionInstance.Threshold = thresholdInstance;
                                }

                                JToken windowSizeValue = conditionValue["WindowSize"];
                                if (windowSizeValue != null)
                                {
                                    TimeSpan windowSizeInstance = TypeConversion.From8601TimeSpan((string)windowSizeValue);
                                    thresholdRuleConditionInstance.WindowSize = windowSizeInstance;
                                }
                                ruleInstance.Condition = thresholdRuleConditionInstance;
                            }
                        }

                        JArray actionsArray = (JArray)responseDoc["Actions"];
                        if (actionsArray != null)
                        {
                            foreach (JToken actionsValue in actionsArray)
                            {
                                string typeName3 = (string)actionsValue["odata.type"];
                                if (typeName3 == "Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models.RuleEmailAction")
                                {
                                    RuleEmailAction ruleEmailActionInstance = new RuleEmailAction();

                                    JToken sendToServiceOwnersValue = actionsValue["SendToServiceOwners"];
                                    if (sendToServiceOwnersValue != null)
                                    {
                                        bool sendToServiceOwnersInstance = (bool)sendToServiceOwnersValue;
                                        ruleEmailActionInstance.SendToServiceOwners = sendToServiceOwnersInstance;
                                    }

                                    JArray customEmailsArray = (JArray)actionsValue["CustomEmails"];
                                    if (customEmailsArray != null)
                                    {
                                        foreach (JToken customEmailsValue in customEmailsArray)
                                        {
                                            ruleEmailActionInstance.CustomEmails.Add((string)customEmailsValue);
                                        }
                                    }
                                    ruleInstance.Actions.Add(ruleEmailActionInstance);
                                }
                            }
                        }

                        JToken lastUpdatedTimeValue = responseDoc["LastUpdatedTime"];
                        if (lastUpdatedTimeValue != null)
                        {
                            DateTime lastUpdatedTimeInstance = (DateTime)lastUpdatedTimeValue;
                            ruleInstance.LastUpdatedTime = lastUpdatedTimeInstance;
                        }
                    }

                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }

        /// <summary>
        /// List the alert rules within a subscription
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List Rules operation response.
        /// </returns>
        public async Task<RuleListResponse> ListAsync(CancellationToken cancellationToken)
        {
            // Validate

            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                Tracing.Enter(invocationId, this, "ListAsync", tracingParameters);
            }

            // Construct URL
            string url = this.Client.BaseUri + "/" + this.Client.Credentials.SubscriptionId + "/services/monitoring/alertrules";

            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);

                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");

                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);

                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }

                    // Create Result
                    RuleListResponse result = new RuleListResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }

                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);

                    if (responseDoc != null)
                    {
                        RuleCollection ruleCollectionInstance = new RuleCollection();
                        result.RuleCollection = ruleCollectionInstance;

                        JArray valueArray = (JArray)responseDoc["Value"];
                        if (valueArray != null)
                        {
                            foreach (JToken valueValue in valueArray)
                            {
                                Rule ruleInstance = new Rule();
                                ruleCollectionInstance.Value.Add(ruleInstance);

                                JToken idValue = valueValue["Id"];
                                if (idValue != null)
                                {
                                    string idInstance = (string)idValue;
                                    ruleInstance.Id = idInstance;
                                }

                                JToken nameValue = valueValue["Name"];
                                if (nameValue != null)
                                {
                                    string nameInstance = (string)nameValue;
                                    ruleInstance.Name = nameInstance;
                                }

                                JToken descriptionValue = valueValue["Description"];
                                if (descriptionValue != null)
                                {
                                    string descriptionInstance = (string)descriptionValue;
                                    ruleInstance.Description = descriptionInstance;
                                }

                                JToken isEnabledValue = valueValue["IsEnabled"];
                                if (isEnabledValue != null)
                                {
                                    bool isEnabledInstance = (bool)isEnabledValue;
                                    ruleInstance.IsEnabled = isEnabledInstance;
                                }

                                JToken conditionValue = valueValue["Condition"];
                                if (conditionValue != null)
                                {
                                    string typeName = (string)conditionValue["odata.type"];
                                    if (typeName == "Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models.ThresholdRuleCondition")
                                    {
                                        ThresholdRuleCondition thresholdRuleConditionInstance = new ThresholdRuleCondition();

                                        JToken dataSourceValue = conditionValue["DataSource"];
                                        if (dataSourceValue != null)
                                        {
                                            string typeName2 = (string)dataSourceValue["odata.type"];
                                            if (typeName2 == "Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models.RuleMetricDataSource")
                                            {
                                                RuleMetricDataSource ruleMetricDataSourceInstance = new RuleMetricDataSource();

                                                JToken resourceIdValue = dataSourceValue["ResourceId"];
                                                if (resourceIdValue != null)
                                                {
                                                    string resourceIdInstance = (string)resourceIdValue;
                                                    ruleMetricDataSourceInstance.ResourceId = resourceIdInstance;
                                                }

                                                JToken metricNamespaceValue = dataSourceValue["MetricNamespace"];
                                                if (metricNamespaceValue != null)
                                                {
                                                    string metricNamespaceInstance = (string)metricNamespaceValue;
                                                    ruleMetricDataSourceInstance.MetricNamespace = metricNamespaceInstance;
                                                }

                                                JToken metricNameValue = dataSourceValue["MetricName"];
                                                if (metricNameValue != null)
                                                {
                                                    string metricNameInstance = (string)metricNameValue;
                                                    ruleMetricDataSourceInstance.MetricName = metricNameInstance;
                                                }
                                                thresholdRuleConditionInstance.DataSource = ruleMetricDataSourceInstance;
                                            }
                                        }

                                        JToken operatorValue = conditionValue["Operator"];
                                        if (operatorValue != null)
                                        {
                                            ConditionOperator operatorInstance = (ConditionOperator)Enum.Parse(typeof(ConditionOperator), (string)operatorValue, false);
                                            thresholdRuleConditionInstance.Operator = operatorInstance;
                                        }

                                        JToken thresholdValue = conditionValue["Threshold"];
                                        if (thresholdValue != null)
                                        {
                                            double thresholdInstance = (double)thresholdValue;
                                            thresholdRuleConditionInstance.Threshold = thresholdInstance;
                                        }

                                        JToken windowSizeValue = conditionValue["WindowSize"];
                                        if (windowSizeValue != null)
                                        {
                                            TimeSpan windowSizeInstance = TypeConversion.From8601TimeSpan((string)windowSizeValue);
                                            thresholdRuleConditionInstance.WindowSize = windowSizeInstance;
                                        }
                                        ruleInstance.Condition = thresholdRuleConditionInstance;
                                    }
                                }

                                JArray actionsArray = (JArray)valueValue["Actions"];
                                if (actionsArray != null)
                                {
                                    foreach (JToken actionsValue in actionsArray)
                                    {
                                        string typeName3 = (string)actionsValue["odata.type"];
                                        if (typeName3 == "Microsoft.WindowsAzure.Management.Monitoring.Alerts.Models.RuleEmailAction")
                                        {
                                            RuleEmailAction ruleEmailActionInstance = new RuleEmailAction();

                                            JToken sendToServiceOwnersValue = actionsValue["SendToServiceOwners"];
                                            if (sendToServiceOwnersValue != null)
                                            {
                                                bool sendToServiceOwnersInstance = (bool)sendToServiceOwnersValue;
                                                ruleEmailActionInstance.SendToServiceOwners = sendToServiceOwnersInstance;
                                            }

                                            JArray customEmailsArray = (JArray)actionsValue["CustomEmails"];
                                            if (customEmailsArray != null)
                                            {
                                                foreach (JToken customEmailsValue in customEmailsArray)
                                                {
                                                    ruleEmailActionInstance.CustomEmails.Add((string)customEmailsValue);
                                                }
                                            }
                                            ruleInstance.Actions.Add(ruleEmailActionInstance);
                                        }
                                    }
                                }

                                JToken lastUpdatedTimeValue = valueValue["LastUpdatedTime"];
                                if (lastUpdatedTimeValue != null)
                                {
                                    DateTime lastUpdatedTimeInstance = (DateTime)lastUpdatedTimeValue;
                                    ruleInstance.LastUpdatedTime = lastUpdatedTimeInstance;
                                }
                            }
                        }
                    }

                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
