// 
// Copyright (c) Microsoft.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Common;
using Microsoft.WindowsAzure.Common.Internals;
using Microsoft.WindowsAzure.Scheduler;
using Microsoft.WindowsAzure.Scheduler.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace Microsoft.WindowsAzure.Scheduler.Models
{
    /// <summary>
    /// A scheduler job.
    /// </summary>
    public partial class Job
    {
        private JobAction _action;
        
        /// <summary>
        /// Action to invoke for the job.
        /// </summary>
        public JobAction Action
        {
            get { return this._action; }
            set { this._action = value; }
        }
        
        private string _id;
        
        /// <summary>
        /// Job identifier.
        /// </summary>
        public string Id
        {
            get { return this._id; }
            set { this._id = value; }
        }
        
        private JobRecurrence _recurrence;
        
        /// <summary>
        /// Recurrence schedule for the job.
        /// </summary>
        public JobRecurrence Recurrence
        {
            get { return this._recurrence; }
            set { this._recurrence = value; }
        }
        
        private System.DateTime? _startTime;
        
        /// <summary>
        /// Start time for the job.  Defined as ISO-8601.
        /// </summary>
        public System.DateTime? StartTime
        {
            get { return this._startTime; }
            set { this._startTime = value; }
        }
        
        private JobState _state;
        
        /// <summary>
        /// Current state of the job.
        /// </summary>
        public JobState State
        {
            get { return this._state; }
            set { this._state = value; }
        }
        
        private JobStatus _status;
        
        /// <summary>
        /// Current status of the job.
        /// </summary>
        public JobStatus Status
        {
            get { return this._status; }
            set { this._status = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the Job class.
        /// </summary>
        public Job()
        {
        }
    }
    
    /// <summary>
    /// Action to invoke for the job.
    /// </summary>
    public partial class JobAction
    {
        private JobErrorAction _errorAction;
        
        /// <summary>
        /// Error Action for the job.
        /// </summary>
        public JobErrorAction ErrorAction
        {
            get { return this._errorAction; }
            set { this._errorAction = value; }
        }
        
        private JobQueueMessage _queueMessage;
        
        /// <summary>
        /// Queue and message for a queue message action type.
        /// </summary>
        public JobQueueMessage QueueMessage
        {
            get { return this._queueMessage; }
            set { this._queueMessage = value; }
        }
        
        private JobHttpRequest _request;
        
        /// <summary>
        /// Request for a http or https action type.
        /// </summary>
        public JobHttpRequest Request
        {
            get { return this._request; }
            set { this._request = value; }
        }
        
        private RetryPolicy _retryPolicy;
        
        /// <summary>
        /// Retry Policy for the job action.
        /// </summary>
        public RetryPolicy RetryPolicy
        {
            get { return this._retryPolicy; }
            set { this._retryPolicy = value; }
        }
        
        private JobActionType _type;
        
        /// <summary>
        /// Type of action. Can be one of http, https, storageQueue.
        /// </summary>
        public JobActionType Type
        {
            get { return this._type; }
            set { this._type = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobAction class.
        /// </summary>
        public JobAction()
        {
        }
    }
    
    /// <summary>
    /// Type of action. Can be one of http, https, storageQueue.
    /// </summary>
    public enum JobActionType
    {
        Http = 0,
        
        Https = 1,
        
        StorageQueue = 2,
    }
    
    /// <summary>
    /// Parameters supplied to the Update Jobs State operation.
    /// </summary>
    public partial class JobCollectionJobsUpdateStateParameters
    {
        private JobState _state;
        
        public JobState State
        {
            get { return this._state; }
            set { this._state = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// JobCollectionJobsUpdateStateParameters class.
        /// </summary>
        public JobCollectionJobsUpdateStateParameters()
        {
        }
    }
    
    /// <summary>
    /// The Update Jobs State operation response.
    /// </summary>
    public partial class JobCollectionJobsUpdateStateResponse : OperationResponse, IEnumerable<Job>
    {
        private IList<Job> _jobs;
        
        /// <summary>
        /// The jobs.
        /// </summary>
        public IList<Job> Jobs
        {
            get { return this._jobs; }
            set { this._jobs = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// JobCollectionJobsUpdateStateResponse class.
        /// </summary>
        public JobCollectionJobsUpdateStateResponse()
        {
            this._jobs = new List<Job>();
        }
        
        /// <summary>
        /// Gets the sequence of Jobs.
        /// </summary>
        public IEnumerator<Job> GetEnumerator()
        {
            return this.Jobs.GetEnumerator();
        }
        
        /// <summary>
        /// Gets the sequence of Jobs.
        /// </summary>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
    }
    
    /// <summary>
    /// Parameters supplied to the Create or Update Job operation.
    /// </summary>
    public partial class JobCreateOrUpdateParameters
    {
        private JobAction _action;
        
        /// <summary>
        /// Action to invoke for the job.
        /// </summary>
        public JobAction Action
        {
            get { return this._action; }
            set { this._action = value; }
        }
        
        private JobRecurrence _recurrence;
        
        /// <summary>
        /// Recurrence schedule for the job.
        /// </summary>
        public JobRecurrence Recurrence
        {
            get { return this._recurrence; }
            set { this._recurrence = value; }
        }
        
        private System.DateTime? _startTime;
        
        /// <summary>
        /// Start time for the job. Defined as ISO-8601.
        /// </summary>
        public System.DateTime? StartTime
        {
            get { return this._startTime; }
            set { this._startTime = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobCreateOrUpdateParameters class.
        /// </summary>
        public JobCreateOrUpdateParameters()
        {
        }
    }
    
    /// <summary>
    /// The Update Job operation response.
    /// </summary>
    public partial class JobCreateOrUpdateResponse : OperationResponse
    {
        private Job _job;
        
        /// <summary>
        /// The updated job.
        /// </summary>
        public Job Job
        {
            get { return this._job; }
            set { this._job = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobCreateOrUpdateResponse class.
        /// </summary>
        public JobCreateOrUpdateResponse()
        {
        }
    }
    
    /// <summary>
    /// Parameters supplied to the Create Job operation.
    /// </summary>
    public partial class JobCreateParameters
    {
        private JobAction _action;
        
        /// <summary>
        /// Action to invoke for the job.
        /// </summary>
        public JobAction Action
        {
            get { return this._action; }
            set { this._action = value; }
        }
        
        private JobRecurrence _recurrence;
        
        /// <summary>
        /// Recurrence schedule for the job.
        /// </summary>
        public JobRecurrence Recurrence
        {
            get { return this._recurrence; }
            set { this._recurrence = value; }
        }
        
        private System.DateTime? _startTime;
        
        /// <summary>
        /// Start time for the job.  Defined as ISO-8601.
        /// </summary>
        public System.DateTime? StartTime
        {
            get { return this._startTime; }
            set { this._startTime = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobCreateParameters class.
        /// </summary>
        public JobCreateParameters()
        {
        }
    }
    
    /// <summary>
    /// The Create Job operation response.
    /// </summary>
    public partial class JobCreateResponse : OperationResponse
    {
        private Job _job;
        
        /// <summary>
        /// The created job.
        /// </summary>
        public Job Job
        {
            get { return this._job; }
            set { this._job = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobCreateResponse class.
        /// </summary>
        public JobCreateResponse()
        {
        }
    }
    
    /// <summary>
    /// Action to invoke for the job error.
    /// </summary>
    public partial class JobErrorAction
    {
        private JobQueueMessage _queueMessage;
        
        /// <summary>
        /// Queue and message for a queue message action type.
        /// </summary>
        public JobQueueMessage QueueMessage
        {
            get { return this._queueMessage; }
            set { this._queueMessage = value; }
        }
        
        private JobHttpRequest _request;
        
        /// <summary>
        /// Request for a http or https action type.
        /// </summary>
        public JobHttpRequest Request
        {
            get { return this._request; }
            set { this._request = value; }
        }
        
        private JobActionType _type;
        
        /// <summary>
        /// Type of action. Can be one of http, https, storageQueue.
        /// </summary>
        public JobActionType Type
        {
            get { return this._type; }
            set { this._type = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobErrorAction class.
        /// </summary>
        public JobErrorAction()
        {
        }
    }
    
    /// <summary>
    /// Parameters supplied to the Get Job History operation.
    /// </summary>
    public partial class JobGetHistoryParameters
    {
        private int? _skip;
        
        /// <summary>
        /// By default the top 100 history entries will be returned in a call
        /// to GET job history. A maximum of 1000 history entries can be
        /// returned in any call.  To get history entries beyond 1000 entries
        /// use $skip.
        /// </summary>
        public int? Skip
        {
            get { return this._skip; }
            set { this._skip = value; }
        }
        
        private Microsoft.WindowsAzure.Scheduler.Models.JobState? _state;
        
        /// <summary>
        /// Filter the job history to have it only return job histories of a
        /// particular state.
        /// </summary>
        public Microsoft.WindowsAzure.Scheduler.Models.JobState? State
        {
            get { return this._state; }
            set { this._state = value; }
        }
        
        private int? _top;
        
        /// <summary>
        /// To receive more or less than the default number of entries in a
        /// call then use the $top OData operation.
        /// </summary>
        public int? Top
        {
            get { return this._top; }
            set { this._top = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobGetHistoryParameters class.
        /// </summary>
        public JobGetHistoryParameters()
        {
        }
    }
    
    /// <summary>
    /// The Get Job History operation response.
    /// </summary>
    public partial class JobGetHistoryResponse : OperationResponse, IEnumerable<JobGetHistoryResponse.JobHistoryEntry>
    {
        private IList<JobGetHistoryResponse.JobHistoryEntry> _jobHistory;
        
        /// <summary>
        /// The job history entries.
        /// </summary>
        public IList<JobGetHistoryResponse.JobHistoryEntry> JobHistory
        {
            get { return this._jobHistory; }
            set { this._jobHistory = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobGetHistoryResponse class.
        /// </summary>
        public JobGetHistoryResponse()
        {
            this._jobHistory = new List<JobGetHistoryResponse.JobHistoryEntry>();
        }
        
        /// <summary>
        /// Gets the sequence of JobHistory.
        /// </summary>
        public IEnumerator<JobGetHistoryResponse.JobHistoryEntry> GetEnumerator()
        {
            return this.JobHistory.GetEnumerator();
        }
        
        /// <summary>
        /// Gets the sequence of JobHistory.
        /// </summary>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
        
        public partial class JobHistoryEntry
        {
            private DateTime _endTime;
            
            public DateTime EndTime
            {
                get { return this._endTime; }
                set { this._endTime = value; }
            }
            
            private string _id;
            
            public string Id
            {
                get { return this._id; }
                set { this._id = value; }
            }
            
            private string _message;
            
            public string Message
            {
                get { return this._message; }
                set { this._message = value; }
            }
            
            private int _recordNumber;
            
            public int RecordNumber
            {
                get { return this._recordNumber; }
                set { this._recordNumber = value; }
            }
            
            private DateTime _startTime;
            
            public DateTime StartTime
            {
                get { return this._startTime; }
                set { this._startTime = value; }
            }
            
            private JobState _state;
            
            public JobState State
            {
                get { return this._state; }
                set { this._state = value; }
            }
            
            private DateTime _timestamp;
            
            public DateTime Timestamp
            {
                get { return this._timestamp; }
                set { this._timestamp = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the JobHistoryEntry class.
            /// </summary>
            public JobHistoryEntry()
            {
            }
        }
    }
    
    /// <summary>
    /// The Get Job operation response.
    /// </summary>
    public partial class JobGetResponse : OperationResponse
    {
        private Job _job;
        
        /// <summary>
        /// The job to get.
        /// </summary>
        public Job Job
        {
            get { return this._job; }
            set { this._job = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobGetResponse class.
        /// </summary>
        public JobGetResponse()
        {
        }
    }
    
    /// <summary>
    /// Request for a http or https action type.
    /// </summary>
    public partial class JobHttpRequest
    {
        private string _body;
        
        /// <summary>
        /// HTTP request body.
        /// </summary>
        public string Body
        {
            get { return this._body; }
            set { this._body = value; }
        }
        
        private IDictionary<string, string> _headers;
        
        /// <summary>
        /// pair of strings representing header name value pairs.
        /// </summary>
        public IDictionary<string, string> Headers
        {
            get { return this._headers; }
            set { this._headers = value; }
        }
        
        private string _method;
        
        /// <summary>
        /// http method e.g. GET, PUT, POST, DELETE.
        /// </summary>
        public string Method
        {
            get { return this._method; }
            set { this._method = value; }
        }
        
        private Uri _uri;
        
        /// <summary>
        /// uri of the endpoint to invoke.
        /// </summary>
        public Uri Uri
        {
            get { return this._uri; }
            set { this._uri = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobHttpRequest class.
        /// </summary>
        public JobHttpRequest()
        {
            this._headers = new Dictionary<string, string>();
        }
    }
    
    /// <summary>
    /// Parameters supplied to the List Jobs operation.
    /// </summary>
    public partial class JobListParameters
    {
        private int? _skip;
        
        /// <summary>
        /// By default the top 1000 jobs will be returned in a call to GET
        /// jobs. A maximum of 1000 jobs can be returned in any call.  To get
        /// jobs beyond 1000 entries use $skip.
        /// </summary>
        public int? Skip
        {
            get { return this._skip; }
            set { this._skip = value; }
        }
        
        private Microsoft.WindowsAzure.Scheduler.Models.JobState? _state;
        
        /// <summary>
        /// To filter the jobs to have it only return jobs of a particular
        /// state use the $filter OData operation with the ‘state’ keyword.
        /// </summary>
        public Microsoft.WindowsAzure.Scheduler.Models.JobState? State
        {
            get { return this._state; }
            set { this._state = value; }
        }
        
        private int? _top;
        
        /// <summary>
        /// To receive more or less than the default number of jobs in a call
        /// then use the $top OData operation.
        /// </summary>
        public int? Top
        {
            get { return this._top; }
            set { this._top = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobListParameters class.
        /// </summary>
        public JobListParameters()
        {
        }
    }
    
    /// <summary>
    /// The List Jobs operation response.
    /// </summary>
    public partial class JobListResponse : OperationResponse, IEnumerable<Job>
    {
        private IList<Job> _jobs;
        
        /// <summary>
        /// The relevant jobs.
        /// </summary>
        public IList<Job> Jobs
        {
            get { return this._jobs; }
            set { this._jobs = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobListResponse class.
        /// </summary>
        public JobListResponse()
        {
            this._jobs = new List<Job>();
        }
        
        /// <summary>
        /// Gets the sequence of Jobs.
        /// </summary>
        public IEnumerator<Job> GetEnumerator()
        {
            return this.Jobs.GetEnumerator();
        }
        
        /// <summary>
        /// Gets the sequence of Jobs.
        /// </summary>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
    }
    
    /// <summary>
    /// Queue and message for a queue message action type.
    /// </summary>
    public partial class JobQueueMessage
    {
        private string _message;
        
        /// <summary>
        /// Message to send to the queue.
        /// </summary>
        public string Message
        {
            get { return this._message; }
            set { this._message = value; }
        }
        
        private string _queueName;
        
        /// <summary>
        /// Name of the queue to send to.
        /// </summary>
        public string QueueName
        {
            get { return this._queueName; }
            set { this._queueName = value; }
        }
        
        private string _sasToken;
        
        /// <summary>
        /// SAS key for the storage account to send message to the queue.
        /// </summary>
        public string SasToken
        {
            get { return this._sasToken; }
            set { this._sasToken = value; }
        }
        
        private string _storageAccountName;
        
        /// <summary>
        /// Name of the storage account for the queue.
        /// </summary>
        public string StorageAccountName
        {
            get { return this._storageAccountName; }
            set { this._storageAccountName = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobQueueMessage class.
        /// </summary>
        public JobQueueMessage()
        {
        }
    }
    
    /// <summary>
    /// Recurrence schedule for the job.
    /// </summary>
    public partial class JobRecurrence
    {
        private int? _count;
        
        /// <summary>
        /// Count of occurrences that will execute. Optional. Default will
        /// recur infinitely
        /// </summary>
        public int? Count
        {
            get { return this._count; }
            set { this._count = value; }
        }
        
        private System.DateTime? _endTime;
        
        /// <summary>
        /// Time in ISO-8601 format after which no more occurrences will
        /// execute.
        /// </summary>
        public System.DateTime? EndTime
        {
            get { return this._endTime; }
            set { this._endTime = value; }
        }
        
        private JobRecurrenceFrequency _frequency;
        
        /// <summary>
        /// The frequency of recurrence.
        /// </summary>
        public JobRecurrenceFrequency Frequency
        {
            get { return this._frequency; }
            set { this._frequency = value; }
        }
        
        private int? _interval;
        
        /// <summary>
        /// Interval of the recurrence at the given frequency.
        /// </summary>
        public int? Interval
        {
            get { return this._interval; }
            set { this._interval = value; }
        }
        
        private JobRecurrenceSchedule _schedule;
        
        /// <summary>
        /// Recurrence schedule for the job.
        /// </summary>
        public JobRecurrenceSchedule Schedule
        {
            get { return this._schedule; }
            set { this._schedule = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobRecurrence class.
        /// </summary>
        public JobRecurrence()
        {
        }
    }
    
    /// <summary>
    /// The frequency of recurrence. Must be one of: minute, hour, day, week,
    /// month, year.
    /// </summary>
    public enum JobRecurrenceFrequency
    {
        Minute = 0,
        
        Hour = 1,
        
        Day = 2,
        
        Week = 3,
        
        Month = 4,
        
        Year = 5,
    }
    
    /// <summary>
    /// Recurrence schedule for the job.
    /// </summary>
    public partial class JobRecurrenceSchedule
    {
        private IList<JobScheduleDay> _days;
        
        /// <summary>
        /// Days of the week that the job should execute on.
        /// </summary>
        public IList<JobScheduleDay> Days
        {
            get { return this._days; }
            set { this._days = value; }
        }
        
        private IList<int> _hours;
        
        /// <summary>
        /// Hours of the day that the job should execute at.
        /// </summary>
        public IList<int> Hours
        {
            get { return this._hours; }
            set { this._hours = value; }
        }
        
        private IList<int> _minutes;
        
        /// <summary>
        /// Minutes of the hour that the job should execute at.
        /// </summary>
        public IList<int> Minutes
        {
            get { return this._minutes; }
            set { this._minutes = value; }
        }
        
        private IList<int> _monthDays;
        
        /// <summary>
        /// Days of the month that the job should execute on.  Must be between
        /// 1 and 31.
        /// </summary>
        public IList<int> MonthDays
        {
            get { return this._monthDays; }
            set { this._monthDays = value; }
        }
        
        private IList<JobScheduleMonthlyOccurrence> _monthlyOccurrences;
        
        /// <summary>
        /// Occurrences of days within a month.
        /// </summary>
        public IList<JobScheduleMonthlyOccurrence> MonthlyOccurrences
        {
            get { return this._monthlyOccurrences; }
            set { this._monthlyOccurrences = value; }
        }
        
        private IList<int> _months;
        
        /// <summary>
        /// Months that the job should execute in.  Must be between 1 and 12.
        /// </summary>
        public IList<int> Months
        {
            get { return this._months; }
            set { this._months = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobRecurrenceSchedule class.
        /// </summary>
        public JobRecurrenceSchedule()
        {
        }
    }
    
    /// <summary>
    /// Must be one of monday, tuesday, wednesday, thursday, friday, saturday,
    /// sunday.
    /// </summary>
    public enum JobScheduleDay
    {
        Monday = 0,
        
        Tuesday = 1,
        
        Wednesday = 2,
        
        Thursday = 3,
        
        Friday = 4,
        
        Saturday = 5,
        
        Sunday = 6,
    }
    
    /// <summary>
    /// Occurrences of days within a month.
    /// </summary>
    public partial class JobScheduleMonthlyOccurrence
    {
        private JobScheduleDay _day;
        
        /// <summary>
        /// Day of the occurrence. Must be one of monday, tuesday, wednesday,
        /// thursday, friday, saturday, sunday.
        /// </summary>
        public JobScheduleDay Day
        {
            get { return this._day; }
            set { this._day = value; }
        }
        
        private int? _occurrence;
        
        /// <summary>
        /// Occurrence of the day within the month. Must be between -31 and 31.
        /// </summary>
        public int? Occurrence
        {
            get { return this._occurrence; }
            set { this._occurrence = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobScheduleMonthlyOccurrence
        /// class.
        /// </summary>
        public JobScheduleMonthlyOccurrence()
        {
        }
    }
    
    public enum JobState
    {
        Enabled = 0,
        
        Disabled = 1,
        
        Faulted = 2,
        
        Completed = 3,
    }
    
    /// <summary>
    /// Current status of the job.
    /// </summary>
    public partial class JobStatus
    {
        private int _executionCount;
        
        /// <summary>
        /// Number of times this job has executed.
        /// </summary>
        public int ExecutionCount
        {
            get { return this._executionCount; }
            set { this._executionCount = value; }
        }
        
        private int _failureCount;
        
        /// <summary>
        /// Number of times this job has failed.
        /// </summary>
        public int FailureCount
        {
            get { return this._failureCount; }
            set { this._failureCount = value; }
        }
        
        private int _faultedCount;
        
        /// <summary>
        /// Number of faulted occurrences (occurrences that were retried and
        /// failed as many times as the retry policy states).
        /// </summary>
        public int FaultedCount
        {
            get { return this._faultedCount; }
            set { this._faultedCount = value; }
        }
        
        private System.DateTime? _lastExecutionTime;
        
        /// <summary>
        /// Time the last occurrence executed in ISO-8601 format. Could be
        /// empty if job has not run yet.
        /// </summary>
        public System.DateTime? LastExecutionTime
        {
            get { return this._lastExecutionTime; }
            set { this._lastExecutionTime = value; }
        }
        
        private System.DateTime? _nextExecutionTime;
        
        /// <summary>
        /// Time of the next occurrence in ISO-8601 format. Could be empty if
        /// the job is completed.
        /// </summary>
        public System.DateTime? NextExecutionTime
        {
            get { return this._nextExecutionTime; }
            set { this._nextExecutionTime = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobStatus class.
        /// </summary>
        public JobStatus()
        {
        }
    }
    
    /// <summary>
    /// Parameters supplied to the Update Job State operation.
    /// </summary>
    public partial class JobUpdateStateParameters
    {
        private JobState _state;
        
        public JobState State
        {
            get { return this._state; }
            set { this._state = value; }
        }
        
        private string _updateStateReason;
        
        public string UpdateStateReason
        {
            get { return this._updateStateReason; }
            set { this._updateStateReason = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobUpdateStateParameters class.
        /// </summary>
        public JobUpdateStateParameters()
        {
        }
    }
    
    /// <summary>
    /// The Update Job State operation response.
    /// </summary>
    public partial class JobUpdateStateResponse : OperationResponse
    {
        private Job _job;
        
        /// <summary>
        /// The updated job.
        /// </summary>
        public Job Job
        {
            get { return this._job; }
            set { this._job = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the JobUpdateStateResponse class.
        /// </summary>
        public JobUpdateStateResponse()
        {
        }
    }
    
    /// <summary>
    /// Retry Policy for the job action.
    /// </summary>
    public partial class RetryPolicy
    {
        private int? _retryCount;
        
        /// <summary>
        /// Number of times a retry should be attempted.
        /// </summary>
        public int? RetryCount
        {
            get { return this._retryCount; }
            set { this._retryCount = value; }
        }
        
        private System.TimeSpan? _retryInterval;
        
        /// <summary>
        /// The interval between retries.
        /// </summary>
        public System.TimeSpan? RetryInterval
        {
            get { return this._retryInterval; }
            set { this._retryInterval = value; }
        }
        
        private RetryType _retryType;
        
        /// <summary>
        /// The Retry Type of the Retry Policy. Can be either  Fixed or None.
        /// </summary>
        public RetryType RetryType
        {
            get { return this._retryType; }
            set { this._retryType = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the RetryPolicy class.
        /// </summary>
        public RetryPolicy()
        {
        }
    }
    
    /// <summary>
    /// The type of Retry Policy
    /// </summary>
    public enum RetryType
    {
        None = 0,
        
        Fixed = 1,
    }
}

namespace Microsoft.WindowsAzure.Scheduler
{
    public partial interface ISchedulerClient
    {
        Uri BaseUri
        {
            get; 
        }
        
        string CloudServiceName
        {
            get; 
        }
        
        SubscriptionCloudCredentials Credentials
        {
            get; 
        }
        
        string JobCollectionName
        {
            get; 
        }
        
        IJobOperations Jobs
        {
            get; 
        }
    }
    
    public static partial class SchedulerClientExtensions
    {
    }
    
    public partial class SchedulerClient : ServiceClient<SchedulerClient>, ISchedulerClient
    {
        private Uri _baseUri;
        
        public Uri BaseUri
        {
            get { return this._baseUri; }
        }
        
        private string _cloudServiceName;
        
        public string CloudServiceName
        {
            get { return this._cloudServiceName; }
        }
        
        private SubscriptionCloudCredentials _credentials;
        
        public SubscriptionCloudCredentials Credentials
        {
            get { return this._credentials; }
        }
        
        private string _jobCollectionName;
        
        public string JobCollectionName
        {
            get { return this._jobCollectionName; }
        }
        
        private IJobOperations _jobs;
        
        public virtual IJobOperations Jobs
        {
            get { return this._jobs; }
        }
        
        /// <summary>
        /// Initializes a new instance of the SchedulerClient class.
        /// </summary>
        private SchedulerClient()
            : base()
        {
            this._jobs = new JobOperations(this);
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the SchedulerClient class.
        /// </summary>
        public SchedulerClient(SubscriptionCloudCredentials credentials, string cloudServiceName, string jobCollectionName, Uri baseUri)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (cloudServiceName == null)
            {
                throw new ArgumentNullException("cloudServiceName");
            }
            if (jobCollectionName == null)
            {
                throw new ArgumentNullException("jobCollectionName");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._credentials = credentials;
            this._cloudServiceName = cloudServiceName;
            this._jobCollectionName = jobCollectionName;
            this._baseUri = baseUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the SchedulerClient class.
        /// </summary>
        public SchedulerClient(SubscriptionCloudCredentials credentials, string cloudServiceName, string jobCollectionName)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (cloudServiceName == null)
            {
                throw new ArgumentNullException("cloudServiceName");
            }
            if (jobCollectionName == null)
            {
                throw new ArgumentNullException("jobCollectionName");
            }
            this._credentials = credentials;
            this._cloudServiceName = cloudServiceName;
            this._jobCollectionName = jobCollectionName;
            this._baseUri = new Uri("https://management.core.windows.net/");
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Parse enum values for type JobActionType.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static JobActionType ParseJobActionType(string value)
        {
            if (value == "http")
            {
                return JobActionType.Http;
            }
            if (value == "https")
            {
                return JobActionType.Https;
            }
            if (value == "storageQueue")
            {
                return JobActionType.StorageQueue;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type JobActionType to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string JobActionTypeToString(JobActionType value)
        {
            if (value == JobActionType.Http)
            {
                return "http";
            }
            if (value == JobActionType.Https)
            {
                return "https";
            }
            if (value == JobActionType.StorageQueue)
            {
                return "storageQueue";
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Parse enum values for type JobRecurrenceFrequency.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static JobRecurrenceFrequency ParseJobRecurrenceFrequency(string value)
        {
            if (value == "minute")
            {
                return JobRecurrenceFrequency.Minute;
            }
            if (value == "hour")
            {
                return JobRecurrenceFrequency.Hour;
            }
            if (value == "day")
            {
                return JobRecurrenceFrequency.Day;
            }
            if (value == "week")
            {
                return JobRecurrenceFrequency.Week;
            }
            if (value == "month")
            {
                return JobRecurrenceFrequency.Month;
            }
            if (value == "year")
            {
                return JobRecurrenceFrequency.Year;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type JobRecurrenceFrequency to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string JobRecurrenceFrequencyToString(JobRecurrenceFrequency value)
        {
            if (value == JobRecurrenceFrequency.Minute)
            {
                return "minute";
            }
            if (value == JobRecurrenceFrequency.Hour)
            {
                return "hour";
            }
            if (value == JobRecurrenceFrequency.Day)
            {
                return "day";
            }
            if (value == JobRecurrenceFrequency.Week)
            {
                return "week";
            }
            if (value == JobRecurrenceFrequency.Month)
            {
                return "month";
            }
            if (value == JobRecurrenceFrequency.Year)
            {
                return "year";
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Parse enum values for type JobScheduleDay.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static JobScheduleDay ParseJobScheduleDay(string value)
        {
            if (value == "monday")
            {
                return JobScheduleDay.Monday;
            }
            if (value == "tuesday")
            {
                return JobScheduleDay.Tuesday;
            }
            if (value == "wednesday")
            {
                return JobScheduleDay.Wednesday;
            }
            if (value == "thursday")
            {
                return JobScheduleDay.Thursday;
            }
            if (value == "friday")
            {
                return JobScheduleDay.Friday;
            }
            if (value == "saturday")
            {
                return JobScheduleDay.Saturday;
            }
            if (value == "sunday")
            {
                return JobScheduleDay.Sunday;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type JobScheduleDay to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string JobScheduleDayToString(JobScheduleDay value)
        {
            if (value == JobScheduleDay.Monday)
            {
                return "monday";
            }
            if (value == JobScheduleDay.Tuesday)
            {
                return "tuesday";
            }
            if (value == JobScheduleDay.Wednesday)
            {
                return "wednesday";
            }
            if (value == JobScheduleDay.Thursday)
            {
                return "thursday";
            }
            if (value == JobScheduleDay.Friday)
            {
                return "friday";
            }
            if (value == JobScheduleDay.Saturday)
            {
                return "saturday";
            }
            if (value == JobScheduleDay.Sunday)
            {
                return "sunday";
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Parse enum values for type JobState.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static JobState ParseJobState(string value)
        {
            if (value == "enabled")
            {
                return JobState.Enabled;
            }
            if (value == "disabled")
            {
                return JobState.Disabled;
            }
            if (value == "faulted")
            {
                return JobState.Faulted;
            }
            if (value == "completed")
            {
                return JobState.Completed;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type JobState to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string JobStateToString(JobState value)
        {
            if (value == JobState.Enabled)
            {
                return "enabled";
            }
            if (value == JobState.Disabled)
            {
                return "disabled";
            }
            if (value == JobState.Faulted)
            {
                return "faulted";
            }
            if (value == JobState.Completed)
            {
                return "completed";
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Parse enum values for type RetryType.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static RetryType ParseRetryType(string value)
        {
            if (value == "none")
            {
                return RetryType.None;
            }
            if (value == "fixed")
            {
                return RetryType.Fixed;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type RetryType to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string RetryTypeToString(RetryType value)
        {
            if (value == RetryType.None)
            {
                return "none";
            }
            if (value == RetryType.Fixed)
            {
                return "fixed";
            }
            throw new ArgumentOutOfRangeException("value");
        }
    }
    
    public partial interface IJobOperations
    {
        /// <summary>
        /// If the user wants the server to create the job id then he can use a
        /// POST request to the jobs resource.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Create Job operation response.
        /// </returns>
        Task<JobCreateResponse> CreateAsync(JobCreateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Jobs can be created or updated with a PUT operation to a job's
        /// address. The resource name is the ID of the job. Doing a PUT
        /// operation on a jobId that already exists will completely overwrite
        /// the existing job. Putting an existing job (replace) will reset
        /// internal execution counters.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to create or update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the job operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Update Job operation response.
        /// </returns>
        Task<JobCreateOrUpdateResponse> CreateOrUpdateAsync(string jobId, JobCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Deletes a job.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to delete.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<OperationResponse> DeleteAsync(string jobId, CancellationToken cancellationToken);
        
        /// <summary>
        /// The status of an individual job can be access by a GET call to a
        /// job's address, jobId.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to get.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Job operation response.
        /// </returns>
        Task<JobGetResponse> GetAsync(string jobId, CancellationToken cancellationToken);
        
        /// <summary>
        /// Job history tracks the updates and executions of a job.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to get the history of.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Get Job History operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Job History operation response.
        /// </returns>
        Task<JobGetHistoryResponse> GetHistoryAsync(string jobId, JobGetHistoryParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Fetch all jobs in a job collection.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the List Jobs operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List Jobs operation response.
        /// </returns>
        Task<JobListResponse> ListAsync(JobListParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Job collections can be updated through a simple PATCH operation.
        /// The format of the request is the same as that for creating a job,
        /// though if a field is unspecified we will carry forward the current
        /// value.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Jobs State operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Update Jobs State operation response.
        /// </returns>
        Task<JobCollectionJobsUpdateStateResponse> UpdateJobCollectionStateAsync(JobCollectionJobsUpdateStateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Jobs can be updated through a simple PATCH operation to a job's
        /// address. The format of the request is the same as that for
        /// creating a job, though if a field is unspecified we will carry
        /// forward the current value.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job State operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Update Job State operation response.
        /// </returns>
        Task<JobUpdateStateResponse> UpdateStateAsync(string jobId, JobUpdateStateParameters parameters, CancellationToken cancellationToken);
    }
    
    public static partial class JobOperationsExtensions
    {
        /// <summary>
        /// If the user wants the server to create the job id then he can use a
        /// POST request to the jobs resource.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job operation.
        /// </param>
        /// <returns>
        /// The Create Job operation response.
        /// </returns>
        public static JobCreateResponse Create(this IJobOperations operations, JobCreateParameters parameters)
        {
            try
            {
                return operations.CreateAsync(parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// If the user wants the server to create the job id then he can use a
        /// POST request to the jobs resource.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job operation.
        /// </param>
        /// <returns>
        /// The Create Job operation response.
        /// </returns>
        public static Task<JobCreateResponse> CreateAsync(this IJobOperations operations, JobCreateParameters parameters)
        {
            return operations.CreateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Jobs can be created or updated with a PUT operation to a job's
        /// address. The resource name is the ID of the job. Doing a PUT
        /// operation on a jobId that already exists will completely overwrite
        /// the existing job. Putting an existing job (replace) will reset
        /// internal execution counters.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to create or update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the job operation.
        /// </param>
        /// <returns>
        /// The Update Job operation response.
        /// </returns>
        public static JobCreateOrUpdateResponse CreateOrUpdate(this IJobOperations operations, string jobId, JobCreateOrUpdateParameters parameters)
        {
            try
            {
                return operations.CreateOrUpdateAsync(jobId, parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Jobs can be created or updated with a PUT operation to a job's
        /// address. The resource name is the ID of the job. Doing a PUT
        /// operation on a jobId that already exists will completely overwrite
        /// the existing job. Putting an existing job (replace) will reset
        /// internal execution counters.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to create or update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the job operation.
        /// </param>
        /// <returns>
        /// The Update Job operation response.
        /// </returns>
        public static Task<JobCreateOrUpdateResponse> CreateOrUpdateAsync(this IJobOperations operations, string jobId, JobCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(jobId, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a job.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse Delete(this IJobOperations operations, string jobId)
        {
            try
            {
                return operations.DeleteAsync(jobId).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Deletes a job.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> DeleteAsync(this IJobOperations operations, string jobId)
        {
            return operations.DeleteAsync(jobId, CancellationToken.None);
        }
        
        /// <summary>
        /// The status of an individual job can be access by a GET call to a
        /// job's address, jobId.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to get.
        /// </param>
        /// <returns>
        /// The Get Job operation response.
        /// </returns>
        public static JobGetResponse Get(this IJobOperations operations, string jobId)
        {
            try
            {
                return operations.GetAsync(jobId).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// The status of an individual job can be access by a GET call to a
        /// job's address, jobId.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to get.
        /// </param>
        /// <returns>
        /// The Get Job operation response.
        /// </returns>
        public static Task<JobGetResponse> GetAsync(this IJobOperations operations, string jobId)
        {
            return operations.GetAsync(jobId, CancellationToken.None);
        }
        
        /// <summary>
        /// Job history tracks the updates and executions of a job.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to get the history of.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Get Job History operation.
        /// </param>
        /// <returns>
        /// The Get Job History operation response.
        /// </returns>
        public static JobGetHistoryResponse GetHistory(this IJobOperations operations, string jobId, JobGetHistoryParameters parameters)
        {
            try
            {
                return operations.GetHistoryAsync(jobId, parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Job history tracks the updates and executions of a job.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to get the history of.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Get Job History operation.
        /// </param>
        /// <returns>
        /// The Get Job History operation response.
        /// </returns>
        public static Task<JobGetHistoryResponse> GetHistoryAsync(this IJobOperations operations, string jobId, JobGetHistoryParameters parameters)
        {
            return operations.GetHistoryAsync(jobId, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Fetch all jobs in a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the List Jobs operation.
        /// </param>
        /// <returns>
        /// The List Jobs operation response.
        /// </returns>
        public static JobListResponse List(this IJobOperations operations, JobListParameters parameters)
        {
            try
            {
                return operations.ListAsync(parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Fetch all jobs in a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the List Jobs operation.
        /// </param>
        /// <returns>
        /// The List Jobs operation response.
        /// </returns>
        public static Task<JobListResponse> ListAsync(this IJobOperations operations, JobListParameters parameters)
        {
            return operations.ListAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Job collections can be updated through a simple PATCH operation.
        /// The format of the request is the same as that for creating a job,
        /// though if a field is unspecified we will carry forward the current
        /// value.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Jobs State operation.
        /// </param>
        /// <returns>
        /// The Update Jobs State operation response.
        /// </returns>
        public static JobCollectionJobsUpdateStateResponse UpdateJobCollectionState(this IJobOperations operations, JobCollectionJobsUpdateStateParameters parameters)
        {
            try
            {
                return operations.UpdateJobCollectionStateAsync(parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Job collections can be updated through a simple PATCH operation.
        /// The format of the request is the same as that for creating a job,
        /// though if a field is unspecified we will carry forward the current
        /// value.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Jobs State operation.
        /// </param>
        /// <returns>
        /// The Update Jobs State operation response.
        /// </returns>
        public static Task<JobCollectionJobsUpdateStateResponse> UpdateJobCollectionStateAsync(this IJobOperations operations, JobCollectionJobsUpdateStateParameters parameters)
        {
            return operations.UpdateJobCollectionStateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Jobs can be updated through a simple PATCH operation to a job's
        /// address. The format of the request is the same as that for
        /// creating a job, though if a field is unspecified we will carry
        /// forward the current value.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job State operation.
        /// </param>
        /// <returns>
        /// The Update Job State operation response.
        /// </returns>
        public static JobUpdateStateResponse UpdateState(this IJobOperations operations, string jobId, JobUpdateStateParameters parameters)
        {
            try
            {
                return operations.UpdateStateAsync(jobId, parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Jobs can be updated through a simple PATCH operation to a job's
        /// address. The format of the request is the same as that for
        /// creating a job, though if a field is unspecified we will carry
        /// forward the current value.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the Microsoft.WindowsAzure.Scheduler.IJobOperations.
        /// </param>
        /// <param name='jobId'>
        /// Id of the job to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job State operation.
        /// </param>
        /// <returns>
        /// The Update Job State operation response.
        /// </returns>
        public static Task<JobUpdateStateResponse> UpdateStateAsync(this IJobOperations operations, string jobId, JobUpdateStateParameters parameters)
        {
            return operations.UpdateStateAsync(jobId, parameters, CancellationToken.None);
        }
    }
    
    internal partial class JobOperations : IServiceOperations<SchedulerClient>, IJobOperations
    {
        /// <summary>
        /// Initializes a new instance of the JobOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal JobOperations(SchedulerClient client)
        {
            this._client = client;
        }
        
        private SchedulerClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Scheduler.SchedulerClient.
        /// </summary>
        public SchedulerClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// If the user wants the server to create the job id then he can use a
        /// POST request to the jobs resource.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Create Job operation response.
        /// </returns>
        public async Task<JobCreateResponse> CreateAsync(JobCreateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Action == null)
            {
                throw new ArgumentNullException("parameters.Action");
            }
            if (parameters.Action.ErrorAction != null)
            {
                if (parameters.Action.ErrorAction.QueueMessage != null)
                {
                    if (parameters.Action.ErrorAction.QueueMessage.Message == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.Message");
                    }
                    if (parameters.Action.ErrorAction.QueueMessage.QueueName == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.QueueName");
                    }
                    if (parameters.Action.ErrorAction.QueueMessage.SasToken == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.SasToken");
                    }
                    if (parameters.Action.ErrorAction.QueueMessage.StorageAccountName == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.StorageAccountName");
                    }
                }
                if (parameters.Action.ErrorAction.Request != null)
                {
                    if (parameters.Action.ErrorAction.Request.Method == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.Request.Method");
                    }
                    if (parameters.Action.ErrorAction.Request.Uri == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.Request.Uri");
                    }
                }
            }
            if (parameters.Action.QueueMessage != null)
            {
                if (parameters.Action.QueueMessage.Message == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.Message");
                }
                if (parameters.Action.QueueMessage.QueueName == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.QueueName");
                }
                if (parameters.Action.QueueMessage.SasToken == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.SasToken");
                }
                if (parameters.Action.QueueMessage.StorageAccountName == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.StorageAccountName");
                }
            }
            if (parameters.Action.Request != null)
            {
                if (parameters.Action.Request.Method == null)
                {
                    throw new ArgumentNullException("parameters.Action.Request.Method");
                }
                if (parameters.Action.Request.Uri == null)
                {
                    throw new ArgumentNullException("parameters.Action.Request.Uri");
                }
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "CreateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs?api-version=2013-10-31_Preview";
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject jobCreateParametersValue = new JObject();
                requestDoc = jobCreateParametersValue;
                
                if (parameters.StartTime != null)
                {
                    JToken startTimeValue = parameters.StartTime;
                    jobCreateParametersValue["startTime"] = startTimeValue;
                }
                
                JObject actionValue = new JObject();
                jobCreateParametersValue["action"] = actionValue;
                
                JToken typeValue = SchedulerClient.JobActionTypeToString(parameters.Action.Type);
                actionValue["type"] = typeValue;
                
                if (parameters.Action.RetryPolicy != null)
                {
                    JObject retryPolicyValue = new JObject();
                    actionValue["retryPolicy"] = retryPolicyValue;
                    
                    JToken retryTypeValue = SchedulerClient.RetryTypeToString(parameters.Action.RetryPolicy.RetryType);
                    retryPolicyValue["retryType"] = retryTypeValue;
                    
                    if (parameters.Action.RetryPolicy.RetryInterval != null)
                    {
                        JToken retryIntervalValue = parameters.Action.RetryPolicy.RetryInterval.ToString();
                        retryPolicyValue["retryInterval"] = retryIntervalValue;
                    }
                    
                    if (parameters.Action.RetryPolicy.RetryCount != null)
                    {
                        JToken retryCountValue = parameters.Action.RetryPolicy.RetryCount;
                        retryPolicyValue["retryCount"] = retryCountValue;
                    }
                }
                
                if (parameters.Action.ErrorAction != null)
                {
                    JObject errorActionValue = new JObject();
                    actionValue["errorAction"] = errorActionValue;
                    
                    JToken typeValue2 = SchedulerClient.JobActionTypeToString(parameters.Action.ErrorAction.Type);
                    errorActionValue["type"] = typeValue2;
                    
                    if (parameters.Action.ErrorAction.Request != null)
                    {
                        JObject requestValue = new JObject();
                        errorActionValue["request"] = requestValue;
                        
                        JToken uriValue = parameters.Action.ErrorAction.Request.Uri.ToString();
                        requestValue["uri"] = uriValue;
                        
                        JToken methodValue = parameters.Action.ErrorAction.Request.Method;
                        requestValue["method"] = methodValue;
                        
                        JObject headersDictionary = new JObject();
                        if (parameters.Action.ErrorAction.Request.Headers != null)
                        {
                            foreach (KeyValuePair<string, string> pair in parameters.Action.ErrorAction.Request.Headers)
                            {
                                string headersKey = pair.Key;
                                string headersValue = pair.Value;
                                headersDictionary[headersKey] = headersValue;
                            }
                        }
                        requestValue["headers"] = headersDictionary;
                        
                        if (parameters.Action.ErrorAction.Request.Body != null)
                        {
                            JToken bodyValue = parameters.Action.ErrorAction.Request.Body;
                            requestValue["body"] = bodyValue;
                        }
                    }
                    
                    if (parameters.Action.ErrorAction.QueueMessage != null)
                    {
                        JObject queueMessageValue = new JObject();
                        errorActionValue["queueMessage"] = queueMessageValue;
                        
                        JToken storageAccountValue = parameters.Action.ErrorAction.QueueMessage.StorageAccountName;
                        queueMessageValue["storageAccount"] = storageAccountValue;
                        
                        JToken queueNameValue = parameters.Action.ErrorAction.QueueMessage.QueueName;
                        queueMessageValue["queueName"] = queueNameValue;
                        
                        JToken sasTokenValue = parameters.Action.ErrorAction.QueueMessage.SasToken;
                        queueMessageValue["sasToken"] = sasTokenValue;
                        
                        JToken messageValue = parameters.Action.ErrorAction.QueueMessage.Message;
                        queueMessageValue["message"] = messageValue;
                    }
                }
                
                if (parameters.Action.Request != null)
                {
                    JObject requestValue2 = new JObject();
                    actionValue["request"] = requestValue2;
                    
                    JToken uriValue2 = parameters.Action.Request.Uri.ToString();
                    requestValue2["uri"] = uriValue2;
                    
                    JToken methodValue2 = parameters.Action.Request.Method;
                    requestValue2["method"] = methodValue2;
                    
                    JObject headersDictionary2 = new JObject();
                    if (parameters.Action.Request.Headers != null)
                    {
                        foreach (KeyValuePair<string, string> pair2 in parameters.Action.Request.Headers)
                        {
                            string headersKey2 = pair2.Key;
                            string headersValue2 = pair2.Value;
                            headersDictionary2[headersKey2] = headersValue2;
                        }
                    }
                    requestValue2["headers"] = headersDictionary2;
                    
                    if (parameters.Action.Request.Body != null)
                    {
                        JToken bodyValue2 = parameters.Action.Request.Body;
                        requestValue2["body"] = bodyValue2;
                    }
                }
                
                if (parameters.Action.QueueMessage != null)
                {
                    JObject queueMessageValue2 = new JObject();
                    actionValue["queueMessage"] = queueMessageValue2;
                    
                    JToken storageAccountValue2 = parameters.Action.QueueMessage.StorageAccountName;
                    queueMessageValue2["storageAccount"] = storageAccountValue2;
                    
                    JToken queueNameValue2 = parameters.Action.QueueMessage.QueueName;
                    queueMessageValue2["queueName"] = queueNameValue2;
                    
                    JToken sasTokenValue2 = parameters.Action.QueueMessage.SasToken;
                    queueMessageValue2["sasToken"] = sasTokenValue2;
                    
                    JToken messageValue2 = parameters.Action.QueueMessage.Message;
                    queueMessageValue2["message"] = messageValue2;
                }
                
                if (parameters.Recurrence != null)
                {
                    JObject recurrenceValue = new JObject();
                    jobCreateParametersValue["recurrence"] = recurrenceValue;
                    
                    JToken frequencyValue = SchedulerClient.JobRecurrenceFrequencyToString(parameters.Recurrence.Frequency);
                    recurrenceValue["frequency"] = frequencyValue;
                    
                    if (parameters.Recurrence.Interval != null)
                    {
                        JToken intervalValue = parameters.Recurrence.Interval;
                        recurrenceValue["interval"] = intervalValue;
                    }
                    
                    if (parameters.Recurrence.Count != null)
                    {
                        JToken countValue = parameters.Recurrence.Count;
                        recurrenceValue["count"] = countValue;
                    }
                    
                    if (parameters.Recurrence.EndTime != null)
                    {
                        JToken endTimeValue = parameters.Recurrence.EndTime;
                        recurrenceValue["endTime"] = endTimeValue;
                    }
                    
                    if (parameters.Recurrence.Schedule != null)
                    {
                        JObject scheduleValue = new JObject();
                        recurrenceValue["schedule"] = scheduleValue;
                        
                        if (parameters.Recurrence.Schedule.Minutes != null)
                        {
                            JArray minutesArray = new JArray();
                            foreach (int minutesItem in parameters.Recurrence.Schedule.Minutes)
                            {
                                minutesArray.Add(minutesItem);
                            }
                            scheduleValue["minutes"] = minutesArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.Hours != null)
                        {
                            JArray hoursArray = new JArray();
                            foreach (int hoursItem in parameters.Recurrence.Schedule.Hours)
                            {
                                hoursArray.Add(hoursItem);
                            }
                            scheduleValue["hours"] = hoursArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.Days != null)
                        {
                            JArray daysArray = new JArray();
                            foreach (JobScheduleDay daysItem in parameters.Recurrence.Schedule.Days)
                            {
                                daysArray.Add(SchedulerClient.JobScheduleDayToString(daysItem));
                            }
                            scheduleValue["days"] = daysArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.Months != null)
                        {
                            JArray monthsArray = new JArray();
                            foreach (int monthsItem in parameters.Recurrence.Schedule.Months)
                            {
                                monthsArray.Add(monthsItem);
                            }
                            scheduleValue["months"] = monthsArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.MonthDays != null)
                        {
                            JArray monthDaysArray = new JArray();
                            foreach (int monthDaysItem in parameters.Recurrence.Schedule.MonthDays)
                            {
                                monthDaysArray.Add(monthDaysItem);
                            }
                            scheduleValue["monthDays"] = monthDaysArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.MonthlyOccurrences != null)
                        {
                            JArray monthlyOccurrencesArray = new JArray();
                            foreach (JobScheduleMonthlyOccurrence monthlyOccurrencesItem in parameters.Recurrence.Schedule.MonthlyOccurrences)
                            {
                                JObject jobScheduleMonthlyOccurrenceValue = new JObject();
                                monthlyOccurrencesArray.Add(jobScheduleMonthlyOccurrenceValue);
                                
                                JToken dayValue = SchedulerClient.JobScheduleDayToString(monthlyOccurrencesItem.Day);
                                jobScheduleMonthlyOccurrenceValue["day"] = dayValue;
                                
                                if (monthlyOccurrencesItem.Occurrence != null)
                                {
                                    JToken occurrenceValue = monthlyOccurrencesItem.Occurrence;
                                    jobScheduleMonthlyOccurrenceValue["occurrence"] = occurrenceValue;
                                }
                            }
                            scheduleValue["monthlyOccurrences"] = monthlyOccurrencesArray;
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobCreateResponse result = new JobCreateResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);
                    
                    if (responseDoc != null)
                    {
                        Job jobInstance = new Job();
                        result.Job = jobInstance;
                        
                        JToken idValue = responseDoc["id"];
                        if (idValue != null)
                        {
                            string idInstance = (string)idValue;
                            jobInstance.Id = idInstance;
                        }
                        
                        JToken startTimeValue2 = responseDoc["startTime"];
                        if (startTimeValue2 != null)
                        {
                            DateTime startTimeInstance = (DateTime)startTimeValue2;
                            jobInstance.StartTime = startTimeInstance;
                        }
                        
                        JToken actionValue2 = responseDoc["action"];
                        if (actionValue2 != null)
                        {
                            JobAction actionInstance = new JobAction();
                            jobInstance.Action = actionInstance;
                            
                            JToken typeValue3 = actionValue2["type"];
                            if (typeValue3 != null)
                            {
                                JobActionType typeInstance = SchedulerClient.ParseJobActionType((string)typeValue3);
                                actionInstance.Type = typeInstance;
                            }
                            
                            JToken retryPolicyValue2 = actionValue2["retryPolicy"];
                            if (retryPolicyValue2 != null)
                            {
                                RetryPolicy retryPolicyInstance = new RetryPolicy();
                                actionInstance.RetryPolicy = retryPolicyInstance;
                                
                                JToken retryTypeValue2 = retryPolicyValue2["retryType"];
                                if (retryTypeValue2 != null)
                                {
                                    RetryType retryTypeInstance = SchedulerClient.ParseRetryType((string)retryTypeValue2);
                                    retryPolicyInstance.RetryType = retryTypeInstance;
                                }
                                
                                JToken retryIntervalValue2 = retryPolicyValue2["retryInterval"];
                                if (retryIntervalValue2 != null)
                                {
                                    TimeSpan retryIntervalInstance = TimeSpan.Parse((string)retryIntervalValue2, CultureInfo.InvariantCulture);
                                    retryPolicyInstance.RetryInterval = retryIntervalInstance;
                                }
                                
                                JToken retryCountValue2 = retryPolicyValue2["retryCount"];
                                if (retryCountValue2 != null)
                                {
                                    int retryCountInstance = (int)retryCountValue2;
                                    retryPolicyInstance.RetryCount = retryCountInstance;
                                }
                            }
                            
                            JToken errorActionValue2 = actionValue2["errorAction"];
                            if (errorActionValue2 != null)
                            {
                                JobErrorAction errorActionInstance = new JobErrorAction();
                                actionInstance.ErrorAction = errorActionInstance;
                                
                                JToken typeValue4 = errorActionValue2["type"];
                                if (typeValue4 != null)
                                {
                                    JobActionType typeInstance2 = SchedulerClient.ParseJobActionType((string)typeValue4);
                                    errorActionInstance.Type = typeInstance2;
                                }
                                
                                JToken requestValue3 = errorActionValue2["request"];
                                if (requestValue3 != null)
                                {
                                    JobHttpRequest requestInstance = new JobHttpRequest();
                                    errorActionInstance.Request = requestInstance;
                                    
                                    JToken uriValue3 = requestValue3["uri"];
                                    if (uriValue3 != null)
                                    {
                                        Uri uriInstance = TypeConversion.TryParseUri((string)uriValue3);
                                        requestInstance.Uri = uriInstance;
                                    }
                                    
                                    JToken methodValue3 = requestValue3["method"];
                                    if (methodValue3 != null)
                                    {
                                        string methodInstance = (string)methodValue3;
                                        requestInstance.Method = methodInstance;
                                    }
                                    
                                    JToken headersSequenceElement = requestValue3["headers"];
                                    if (headersSequenceElement != null)
                                    {
                                        foreach (JProperty property in headersSequenceElement)
                                        {
                                            string headersKey3 = (string)property.Name;
                                            string headersValue3 = (string)property.Value;
                                            requestInstance.Headers.Add(headersKey3, headersValue3);
                                        }
                                    }
                                    
                                    JToken bodyValue3 = requestValue3["body"];
                                    if (bodyValue3 != null)
                                    {
                                        string bodyInstance = (string)bodyValue3;
                                        requestInstance.Body = bodyInstance;
                                    }
                                }
                                
                                JToken queueMessageValue3 = errorActionValue2["queueMessage"];
                                if (queueMessageValue3 != null)
                                {
                                    JobQueueMessage queueMessageInstance = new JobQueueMessage();
                                    errorActionInstance.QueueMessage = queueMessageInstance;
                                    
                                    JToken storageAccountValue3 = queueMessageValue3["storageAccount"];
                                    if (storageAccountValue3 != null)
                                    {
                                        string storageAccountInstance = (string)storageAccountValue3;
                                        queueMessageInstance.StorageAccountName = storageAccountInstance;
                                    }
                                    
                                    JToken queueNameValue3 = queueMessageValue3["queueName"];
                                    if (queueNameValue3 != null)
                                    {
                                        string queueNameInstance = (string)queueNameValue3;
                                        queueMessageInstance.QueueName = queueNameInstance;
                                    }
                                    
                                    JToken sasTokenValue3 = queueMessageValue3["sasToken"];
                                    if (sasTokenValue3 != null)
                                    {
                                        string sasTokenInstance = (string)sasTokenValue3;
                                        queueMessageInstance.SasToken = sasTokenInstance;
                                    }
                                    
                                    JToken messageValue3 = queueMessageValue3["message"];
                                    if (messageValue3 != null)
                                    {
                                        string messageInstance = (string)messageValue3;
                                        queueMessageInstance.Message = messageInstance;
                                    }
                                }
                            }
                            
                            JToken requestValue4 = actionValue2["request"];
                            if (requestValue4 != null)
                            {
                                JobHttpRequest requestInstance2 = new JobHttpRequest();
                                actionInstance.Request = requestInstance2;
                                
                                JToken uriValue4 = requestValue4["uri"];
                                if (uriValue4 != null)
                                {
                                    Uri uriInstance2 = TypeConversion.TryParseUri((string)uriValue4);
                                    requestInstance2.Uri = uriInstance2;
                                }
                                
                                JToken methodValue4 = requestValue4["method"];
                                if (methodValue4 != null)
                                {
                                    string methodInstance2 = (string)methodValue4;
                                    requestInstance2.Method = methodInstance2;
                                }
                                
                                JToken headersSequenceElement2 = requestValue4["headers"];
                                if (headersSequenceElement2 != null)
                                {
                                    foreach (JProperty property2 in headersSequenceElement2)
                                    {
                                        string headersKey4 = (string)property2.Name;
                                        string headersValue4 = (string)property2.Value;
                                        requestInstance2.Headers.Add(headersKey4, headersValue4);
                                    }
                                }
                                
                                JToken bodyValue4 = requestValue4["body"];
                                if (bodyValue4 != null)
                                {
                                    string bodyInstance2 = (string)bodyValue4;
                                    requestInstance2.Body = bodyInstance2;
                                }
                            }
                            
                            JToken queueMessageValue4 = actionValue2["queueMessage"];
                            if (queueMessageValue4 != null)
                            {
                                JobQueueMessage queueMessageInstance2 = new JobQueueMessage();
                                actionInstance.QueueMessage = queueMessageInstance2;
                                
                                JToken storageAccountValue4 = queueMessageValue4["storageAccount"];
                                if (storageAccountValue4 != null)
                                {
                                    string storageAccountInstance2 = (string)storageAccountValue4;
                                    queueMessageInstance2.StorageAccountName = storageAccountInstance2;
                                }
                                
                                JToken queueNameValue4 = queueMessageValue4["queueName"];
                                if (queueNameValue4 != null)
                                {
                                    string queueNameInstance2 = (string)queueNameValue4;
                                    queueMessageInstance2.QueueName = queueNameInstance2;
                                }
                                
                                JToken sasTokenValue4 = queueMessageValue4["sasToken"];
                                if (sasTokenValue4 != null)
                                {
                                    string sasTokenInstance2 = (string)sasTokenValue4;
                                    queueMessageInstance2.SasToken = sasTokenInstance2;
                                }
                                
                                JToken messageValue4 = queueMessageValue4["message"];
                                if (messageValue4 != null)
                                {
                                    string messageInstance2 = (string)messageValue4;
                                    queueMessageInstance2.Message = messageInstance2;
                                }
                            }
                        }
                        
                        JToken recurrenceValue2 = responseDoc["recurrence"];
                        if (recurrenceValue2 != null)
                        {
                            JobRecurrence recurrenceInstance = new JobRecurrence();
                            jobInstance.Recurrence = recurrenceInstance;
                            
                            JToken frequencyValue2 = recurrenceValue2["frequency"];
                            if (frequencyValue2 != null)
                            {
                                JobRecurrenceFrequency frequencyInstance = SchedulerClient.ParseJobRecurrenceFrequency((string)frequencyValue2);
                                recurrenceInstance.Frequency = frequencyInstance;
                            }
                            
                            JToken intervalValue2 = recurrenceValue2["interval"];
                            if (intervalValue2 != null)
                            {
                                int intervalInstance = (int)intervalValue2;
                                recurrenceInstance.Interval = intervalInstance;
                            }
                            
                            JToken countValue2 = recurrenceValue2["count"];
                            if (countValue2 != null)
                            {
                                int countInstance = (int)countValue2;
                                recurrenceInstance.Count = countInstance;
                            }
                            
                            JToken endTimeValue2 = recurrenceValue2["endTime"];
                            if (endTimeValue2 != null)
                            {
                                DateTime endTimeInstance = (DateTime)endTimeValue2;
                                recurrenceInstance.EndTime = endTimeInstance;
                            }
                            
                            JToken scheduleValue2 = recurrenceValue2["schedule"];
                            if (scheduleValue2 != null)
                            {
                                JobRecurrenceSchedule scheduleInstance = new JobRecurrenceSchedule();
                                recurrenceInstance.Schedule = scheduleInstance;
                                
                                JArray minutesArray2 = (JArray)scheduleValue2["minutes"];
                                if (minutesArray2 != null)
                                {
                                    foreach (JToken minutesValue in minutesArray2)
                                    {
                                        scheduleInstance.Minutes.Add((int)minutesValue);
                                    }
                                }
                                
                                JArray hoursArray2 = (JArray)scheduleValue2["hours"];
                                if (hoursArray2 != null)
                                {
                                    foreach (JToken hoursValue in hoursArray2)
                                    {
                                        scheduleInstance.Hours.Add((int)hoursValue);
                                    }
                                }
                                
                                JArray daysArray2 = (JArray)scheduleValue2["days"];
                                if (daysArray2 != null)
                                {
                                    foreach (JToken daysValue in daysArray2)
                                    {
                                        scheduleInstance.Days.Add(SchedulerClient.ParseJobScheduleDay((string)daysValue));
                                    }
                                }
                                
                                JArray monthsArray2 = (JArray)scheduleValue2["months"];
                                if (monthsArray2 != null)
                                {
                                    foreach (JToken monthsValue in monthsArray2)
                                    {
                                        scheduleInstance.Months.Add((int)monthsValue);
                                    }
                                }
                                
                                JArray monthDaysArray2 = (JArray)scheduleValue2["monthDays"];
                                if (monthDaysArray2 != null)
                                {
                                    foreach (JToken monthDaysValue in monthDaysArray2)
                                    {
                                        scheduleInstance.MonthDays.Add((int)monthDaysValue);
                                    }
                                }
                                
                                JArray monthlyOccurrencesArray2 = (JArray)scheduleValue2["monthlyOccurrences"];
                                if (monthlyOccurrencesArray2 != null)
                                {
                                    foreach (JToken monthlyOccurrencesValue in monthlyOccurrencesArray2)
                                    {
                                        JobScheduleMonthlyOccurrence jobScheduleMonthlyOccurrenceInstance = new JobScheduleMonthlyOccurrence();
                                        scheduleInstance.MonthlyOccurrences.Add(jobScheduleMonthlyOccurrenceInstance);
                                        
                                        JToken dayValue2 = monthlyOccurrencesValue["day"];
                                        if (dayValue2 != null)
                                        {
                                            JobScheduleDay dayInstance = SchedulerClient.ParseJobScheduleDay((string)dayValue2);
                                            jobScheduleMonthlyOccurrenceInstance.Day = dayInstance;
                                        }
                                        
                                        JToken occurrenceValue2 = monthlyOccurrencesValue["occurrence"];
                                        if (occurrenceValue2 != null)
                                        {
                                            int occurrenceInstance = (int)occurrenceValue2;
                                            jobScheduleMonthlyOccurrenceInstance.Occurrence = occurrenceInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        JToken statusValue = responseDoc["status"];
                        if (statusValue != null)
                        {
                            JobStatus statusInstance = new JobStatus();
                            jobInstance.Status = statusInstance;
                            
                            JToken lastExecutionTimeValue = statusValue["lastExecutionTime"];
                            if (lastExecutionTimeValue != null)
                            {
                                DateTime lastExecutionTimeInstance = (DateTime)lastExecutionTimeValue;
                                statusInstance.LastExecutionTime = lastExecutionTimeInstance;
                            }
                            
                            JToken nextExecutionTimeValue = statusValue["nextExecutionTime"];
                            if (nextExecutionTimeValue != null)
                            {
                                DateTime nextExecutionTimeInstance = (DateTime)nextExecutionTimeValue;
                                statusInstance.NextExecutionTime = nextExecutionTimeInstance;
                            }
                            
                            JToken executionCountValue = statusValue["executionCount"];
                            if (executionCountValue != null)
                            {
                                int executionCountInstance = (int)executionCountValue;
                                statusInstance.ExecutionCount = executionCountInstance;
                            }
                            
                            JToken failureCountValue = statusValue["failureCount"];
                            if (failureCountValue != null)
                            {
                                int failureCountInstance = (int)failureCountValue;
                                statusInstance.FailureCount = failureCountInstance;
                            }
                            
                            JToken faultedCountValue = statusValue["faultedCount"];
                            if (faultedCountValue != null)
                            {
                                int faultedCountInstance = (int)faultedCountValue;
                                statusInstance.FaultedCount = faultedCountInstance;
                            }
                        }
                        
                        JToken stateValue = responseDoc["state"];
                        if (stateValue != null)
                        {
                            JobState stateInstance = SchedulerClient.ParseJobState((string)stateValue);
                            jobInstance.State = stateInstance;
                        }
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Jobs can be created or updated with a PUT operation to a job's
        /// address. The resource name is the ID of the job. Doing a PUT
        /// operation on a jobId that already exists will completely overwrite
        /// the existing job. Putting an existing job (replace) will reset
        /// internal execution counters.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to create or update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the job operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Update Job operation response.
        /// </returns>
        public async Task<JobCreateOrUpdateResponse> CreateOrUpdateAsync(string jobId, JobCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (jobId == null)
            {
                throw new ArgumentNullException("jobId");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Action == null)
            {
                throw new ArgumentNullException("parameters.Action");
            }
            if (parameters.Action.ErrorAction != null)
            {
                if (parameters.Action.ErrorAction.QueueMessage != null)
                {
                    if (parameters.Action.ErrorAction.QueueMessage.Message == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.Message");
                    }
                    if (parameters.Action.ErrorAction.QueueMessage.QueueName == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.QueueName");
                    }
                    if (parameters.Action.ErrorAction.QueueMessage.SasToken == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.SasToken");
                    }
                    if (parameters.Action.ErrorAction.QueueMessage.StorageAccountName == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.QueueMessage.StorageAccountName");
                    }
                }
                if (parameters.Action.ErrorAction.Request != null)
                {
                    if (parameters.Action.ErrorAction.Request.Method == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.Request.Method");
                    }
                    if (parameters.Action.ErrorAction.Request.Uri == null)
                    {
                        throw new ArgumentNullException("parameters.Action.ErrorAction.Request.Uri");
                    }
                }
            }
            if (parameters.Action.QueueMessage != null)
            {
                if (parameters.Action.QueueMessage.Message == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.Message");
                }
                if (parameters.Action.QueueMessage.QueueName == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.QueueName");
                }
                if (parameters.Action.QueueMessage.SasToken == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.SasToken");
                }
                if (parameters.Action.QueueMessage.StorageAccountName == null)
                {
                    throw new ArgumentNullException("parameters.Action.QueueMessage.StorageAccountName");
                }
            }
            if (parameters.Action.Request != null)
            {
                if (parameters.Action.Request.Method == null)
                {
                    throw new ArgumentNullException("parameters.Action.Request.Method");
                }
                if (parameters.Action.Request.Uri == null)
                {
                    throw new ArgumentNullException("parameters.Action.Request.Uri");
                }
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs/" + jobId + "?api-version=2013-10-31_Preview";
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject jobCreateOrUpdateParametersValue = new JObject();
                requestDoc = jobCreateOrUpdateParametersValue;
                
                if (parameters.StartTime != null)
                {
                    JToken startTimeValue = parameters.StartTime;
                    jobCreateOrUpdateParametersValue["startTime"] = startTimeValue;
                }
                
                JObject actionValue = new JObject();
                jobCreateOrUpdateParametersValue["action"] = actionValue;
                
                JToken typeValue = SchedulerClient.JobActionTypeToString(parameters.Action.Type);
                actionValue["type"] = typeValue;
                
                if (parameters.Action.RetryPolicy != null)
                {
                    JObject retryPolicyValue = new JObject();
                    actionValue["retryPolicy"] = retryPolicyValue;
                    
                    JToken retryTypeValue = SchedulerClient.RetryTypeToString(parameters.Action.RetryPolicy.RetryType);
                    retryPolicyValue["retryType"] = retryTypeValue;
                    
                    if (parameters.Action.RetryPolicy.RetryInterval != null)
                    {
                        JToken retryIntervalValue = parameters.Action.RetryPolicy.RetryInterval.ToString();
                        retryPolicyValue["retryInterval"] = retryIntervalValue;
                    }
                    
                    if (parameters.Action.RetryPolicy.RetryCount != null)
                    {
                        JToken retryCountValue = parameters.Action.RetryPolicy.RetryCount;
                        retryPolicyValue["retryCount"] = retryCountValue;
                    }
                }
                
                if (parameters.Action.ErrorAction != null)
                {
                    JObject errorActionValue = new JObject();
                    actionValue["errorAction"] = errorActionValue;
                    
                    JToken typeValue2 = SchedulerClient.JobActionTypeToString(parameters.Action.ErrorAction.Type);
                    errorActionValue["type"] = typeValue2;
                    
                    if (parameters.Action.ErrorAction.Request != null)
                    {
                        JObject requestValue = new JObject();
                        errorActionValue["request"] = requestValue;
                        
                        JToken uriValue = parameters.Action.ErrorAction.Request.Uri.ToString();
                        requestValue["uri"] = uriValue;
                        
                        JToken methodValue = parameters.Action.ErrorAction.Request.Method;
                        requestValue["method"] = methodValue;
                        
                        JObject headersDictionary = new JObject();
                        if (parameters.Action.ErrorAction.Request.Headers != null)
                        {
                            foreach (KeyValuePair<string, string> pair in parameters.Action.ErrorAction.Request.Headers)
                            {
                                string headersKey = pair.Key;
                                string headersValue = pair.Value;
                                headersDictionary[headersKey] = headersValue;
                            }
                        }
                        requestValue["headers"] = headersDictionary;
                        
                        if (parameters.Action.ErrorAction.Request.Body != null)
                        {
                            JToken bodyValue = parameters.Action.ErrorAction.Request.Body;
                            requestValue["body"] = bodyValue;
                        }
                    }
                    
                    if (parameters.Action.ErrorAction.QueueMessage != null)
                    {
                        JObject queueMessageValue = new JObject();
                        errorActionValue["queueMessage"] = queueMessageValue;
                        
                        JToken storageAccountValue = parameters.Action.ErrorAction.QueueMessage.StorageAccountName;
                        queueMessageValue["storageAccount"] = storageAccountValue;
                        
                        JToken queueNameValue = parameters.Action.ErrorAction.QueueMessage.QueueName;
                        queueMessageValue["queueName"] = queueNameValue;
                        
                        JToken sasTokenValue = parameters.Action.ErrorAction.QueueMessage.SasToken;
                        queueMessageValue["sasToken"] = sasTokenValue;
                        
                        JToken messageValue = parameters.Action.ErrorAction.QueueMessage.Message;
                        queueMessageValue["message"] = messageValue;
                    }
                }
                
                if (parameters.Action.Request != null)
                {
                    JObject requestValue2 = new JObject();
                    actionValue["request"] = requestValue2;
                    
                    JToken uriValue2 = parameters.Action.Request.Uri.ToString();
                    requestValue2["uri"] = uriValue2;
                    
                    JToken methodValue2 = parameters.Action.Request.Method;
                    requestValue2["method"] = methodValue2;
                    
                    JObject headersDictionary2 = new JObject();
                    if (parameters.Action.Request.Headers != null)
                    {
                        foreach (KeyValuePair<string, string> pair2 in parameters.Action.Request.Headers)
                        {
                            string headersKey2 = pair2.Key;
                            string headersValue2 = pair2.Value;
                            headersDictionary2[headersKey2] = headersValue2;
                        }
                    }
                    requestValue2["headers"] = headersDictionary2;
                    
                    if (parameters.Action.Request.Body != null)
                    {
                        JToken bodyValue2 = parameters.Action.Request.Body;
                        requestValue2["body"] = bodyValue2;
                    }
                }
                
                if (parameters.Action.QueueMessage != null)
                {
                    JObject queueMessageValue2 = new JObject();
                    actionValue["queueMessage"] = queueMessageValue2;
                    
                    JToken storageAccountValue2 = parameters.Action.QueueMessage.StorageAccountName;
                    queueMessageValue2["storageAccount"] = storageAccountValue2;
                    
                    JToken queueNameValue2 = parameters.Action.QueueMessage.QueueName;
                    queueMessageValue2["queueName"] = queueNameValue2;
                    
                    JToken sasTokenValue2 = parameters.Action.QueueMessage.SasToken;
                    queueMessageValue2["sasToken"] = sasTokenValue2;
                    
                    JToken messageValue2 = parameters.Action.QueueMessage.Message;
                    queueMessageValue2["message"] = messageValue2;
                }
                
                if (parameters.Recurrence != null)
                {
                    JObject recurrenceValue = new JObject();
                    jobCreateOrUpdateParametersValue["recurrence"] = recurrenceValue;
                    
                    JToken frequencyValue = SchedulerClient.JobRecurrenceFrequencyToString(parameters.Recurrence.Frequency);
                    recurrenceValue["frequency"] = frequencyValue;
                    
                    if (parameters.Recurrence.Interval != null)
                    {
                        JToken intervalValue = parameters.Recurrence.Interval;
                        recurrenceValue["interval"] = intervalValue;
                    }
                    
                    if (parameters.Recurrence.Count != null)
                    {
                        JToken countValue = parameters.Recurrence.Count;
                        recurrenceValue["count"] = countValue;
                    }
                    
                    if (parameters.Recurrence.EndTime != null)
                    {
                        JToken endTimeValue = parameters.Recurrence.EndTime;
                        recurrenceValue["endTime"] = endTimeValue;
                    }
                    
                    if (parameters.Recurrence.Schedule != null)
                    {
                        JObject scheduleValue = new JObject();
                        recurrenceValue["schedule"] = scheduleValue;
                        
                        if (parameters.Recurrence.Schedule.Minutes != null)
                        {
                            JArray minutesArray = new JArray();
                            foreach (int minutesItem in parameters.Recurrence.Schedule.Minutes)
                            {
                                minutesArray.Add(minutesItem);
                            }
                            scheduleValue["minutes"] = minutesArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.Hours != null)
                        {
                            JArray hoursArray = new JArray();
                            foreach (int hoursItem in parameters.Recurrence.Schedule.Hours)
                            {
                                hoursArray.Add(hoursItem);
                            }
                            scheduleValue["hours"] = hoursArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.Days != null)
                        {
                            JArray daysArray = new JArray();
                            foreach (JobScheduleDay daysItem in parameters.Recurrence.Schedule.Days)
                            {
                                daysArray.Add(SchedulerClient.JobScheduleDayToString(daysItem));
                            }
                            scheduleValue["days"] = daysArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.Months != null)
                        {
                            JArray monthsArray = new JArray();
                            foreach (int monthsItem in parameters.Recurrence.Schedule.Months)
                            {
                                monthsArray.Add(monthsItem);
                            }
                            scheduleValue["months"] = monthsArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.MonthDays != null)
                        {
                            JArray monthDaysArray = new JArray();
                            foreach (int monthDaysItem in parameters.Recurrence.Schedule.MonthDays)
                            {
                                monthDaysArray.Add(monthDaysItem);
                            }
                            scheduleValue["monthDays"] = monthDaysArray;
                        }
                        
                        if (parameters.Recurrence.Schedule.MonthlyOccurrences != null)
                        {
                            JArray monthlyOccurrencesArray = new JArray();
                            foreach (JobScheduleMonthlyOccurrence monthlyOccurrencesItem in parameters.Recurrence.Schedule.MonthlyOccurrences)
                            {
                                JObject jobScheduleMonthlyOccurrenceValue = new JObject();
                                monthlyOccurrencesArray.Add(jobScheduleMonthlyOccurrenceValue);
                                
                                JToken dayValue = SchedulerClient.JobScheduleDayToString(monthlyOccurrencesItem.Day);
                                jobScheduleMonthlyOccurrenceValue["day"] = dayValue;
                                
                                if (monthlyOccurrencesItem.Occurrence != null)
                                {
                                    JToken occurrenceValue = monthlyOccurrencesItem.Occurrence;
                                    jobScheduleMonthlyOccurrenceValue["occurrence"] = occurrenceValue;
                                }
                            }
                            scheduleValue["monthlyOccurrences"] = monthlyOccurrencesArray;
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobCreateOrUpdateResponse result = new JobCreateOrUpdateResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);
                    
                    if (responseDoc != null)
                    {
                        Job jobInstance = new Job();
                        result.Job = jobInstance;
                        
                        JToken idValue = responseDoc["id"];
                        if (idValue != null)
                        {
                            string idInstance = (string)idValue;
                            jobInstance.Id = idInstance;
                        }
                        
                        JToken startTimeValue2 = responseDoc["startTime"];
                        if (startTimeValue2 != null)
                        {
                            DateTime startTimeInstance = (DateTime)startTimeValue2;
                            jobInstance.StartTime = startTimeInstance;
                        }
                        
                        JToken actionValue2 = responseDoc["action"];
                        if (actionValue2 != null)
                        {
                            JobAction actionInstance = new JobAction();
                            jobInstance.Action = actionInstance;
                            
                            JToken typeValue3 = actionValue2["type"];
                            if (typeValue3 != null)
                            {
                                JobActionType typeInstance = SchedulerClient.ParseJobActionType((string)typeValue3);
                                actionInstance.Type = typeInstance;
                            }
                            
                            JToken retryPolicyValue2 = actionValue2["retryPolicy"];
                            if (retryPolicyValue2 != null)
                            {
                                RetryPolicy retryPolicyInstance = new RetryPolicy();
                                actionInstance.RetryPolicy = retryPolicyInstance;
                                
                                JToken retryTypeValue2 = retryPolicyValue2["retryType"];
                                if (retryTypeValue2 != null)
                                {
                                    RetryType retryTypeInstance = SchedulerClient.ParseRetryType((string)retryTypeValue2);
                                    retryPolicyInstance.RetryType = retryTypeInstance;
                                }
                                
                                JToken retryIntervalValue2 = retryPolicyValue2["retryInterval"];
                                if (retryIntervalValue2 != null)
                                {
                                    TimeSpan retryIntervalInstance = TimeSpan.Parse((string)retryIntervalValue2, CultureInfo.InvariantCulture);
                                    retryPolicyInstance.RetryInterval = retryIntervalInstance;
                                }
                                
                                JToken retryCountValue2 = retryPolicyValue2["retryCount"];
                                if (retryCountValue2 != null)
                                {
                                    int retryCountInstance = (int)retryCountValue2;
                                    retryPolicyInstance.RetryCount = retryCountInstance;
                                }
                            }
                            
                            JToken errorActionValue2 = actionValue2["errorAction"];
                            if (errorActionValue2 != null)
                            {
                                JobErrorAction errorActionInstance = new JobErrorAction();
                                actionInstance.ErrorAction = errorActionInstance;
                                
                                JToken typeValue4 = errorActionValue2["type"];
                                if (typeValue4 != null)
                                {
                                    JobActionType typeInstance2 = SchedulerClient.ParseJobActionType((string)typeValue4);
                                    errorActionInstance.Type = typeInstance2;
                                }
                                
                                JToken requestValue3 = errorActionValue2["request"];
                                if (requestValue3 != null)
                                {
                                    JobHttpRequest requestInstance = new JobHttpRequest();
                                    errorActionInstance.Request = requestInstance;
                                    
                                    JToken uriValue3 = requestValue3["uri"];
                                    if (uriValue3 != null)
                                    {
                                        Uri uriInstance = TypeConversion.TryParseUri((string)uriValue3);
                                        requestInstance.Uri = uriInstance;
                                    }
                                    
                                    JToken methodValue3 = requestValue3["method"];
                                    if (methodValue3 != null)
                                    {
                                        string methodInstance = (string)methodValue3;
                                        requestInstance.Method = methodInstance;
                                    }
                                    
                                    JToken headersSequenceElement = requestValue3["headers"];
                                    if (headersSequenceElement != null)
                                    {
                                        foreach (JProperty property in headersSequenceElement)
                                        {
                                            string headersKey3 = (string)property.Name;
                                            string headersValue3 = (string)property.Value;
                                            requestInstance.Headers.Add(headersKey3, headersValue3);
                                        }
                                    }
                                    
                                    JToken bodyValue3 = requestValue3["body"];
                                    if (bodyValue3 != null)
                                    {
                                        string bodyInstance = (string)bodyValue3;
                                        requestInstance.Body = bodyInstance;
                                    }
                                }
                                
                                JToken queueMessageValue3 = errorActionValue2["queueMessage"];
                                if (queueMessageValue3 != null)
                                {
                                    JobQueueMessage queueMessageInstance = new JobQueueMessage();
                                    errorActionInstance.QueueMessage = queueMessageInstance;
                                    
                                    JToken storageAccountValue3 = queueMessageValue3["storageAccount"];
                                    if (storageAccountValue3 != null)
                                    {
                                        string storageAccountInstance = (string)storageAccountValue3;
                                        queueMessageInstance.StorageAccountName = storageAccountInstance;
                                    }
                                    
                                    JToken queueNameValue3 = queueMessageValue3["queueName"];
                                    if (queueNameValue3 != null)
                                    {
                                        string queueNameInstance = (string)queueNameValue3;
                                        queueMessageInstance.QueueName = queueNameInstance;
                                    }
                                    
                                    JToken sasTokenValue3 = queueMessageValue3["sasToken"];
                                    if (sasTokenValue3 != null)
                                    {
                                        string sasTokenInstance = (string)sasTokenValue3;
                                        queueMessageInstance.SasToken = sasTokenInstance;
                                    }
                                    
                                    JToken messageValue3 = queueMessageValue3["message"];
                                    if (messageValue3 != null)
                                    {
                                        string messageInstance = (string)messageValue3;
                                        queueMessageInstance.Message = messageInstance;
                                    }
                                }
                            }
                            
                            JToken requestValue4 = actionValue2["request"];
                            if (requestValue4 != null)
                            {
                                JobHttpRequest requestInstance2 = new JobHttpRequest();
                                actionInstance.Request = requestInstance2;
                                
                                JToken uriValue4 = requestValue4["uri"];
                                if (uriValue4 != null)
                                {
                                    Uri uriInstance2 = TypeConversion.TryParseUri((string)uriValue4);
                                    requestInstance2.Uri = uriInstance2;
                                }
                                
                                JToken methodValue4 = requestValue4["method"];
                                if (methodValue4 != null)
                                {
                                    string methodInstance2 = (string)methodValue4;
                                    requestInstance2.Method = methodInstance2;
                                }
                                
                                JToken headersSequenceElement2 = requestValue4["headers"];
                                if (headersSequenceElement2 != null)
                                {
                                    foreach (JProperty property2 in headersSequenceElement2)
                                    {
                                        string headersKey4 = (string)property2.Name;
                                        string headersValue4 = (string)property2.Value;
                                        requestInstance2.Headers.Add(headersKey4, headersValue4);
                                    }
                                }
                                
                                JToken bodyValue4 = requestValue4["body"];
                                if (bodyValue4 != null)
                                {
                                    string bodyInstance2 = (string)bodyValue4;
                                    requestInstance2.Body = bodyInstance2;
                                }
                            }
                            
                            JToken queueMessageValue4 = actionValue2["queueMessage"];
                            if (queueMessageValue4 != null)
                            {
                                JobQueueMessage queueMessageInstance2 = new JobQueueMessage();
                                actionInstance.QueueMessage = queueMessageInstance2;
                                
                                JToken storageAccountValue4 = queueMessageValue4["storageAccount"];
                                if (storageAccountValue4 != null)
                                {
                                    string storageAccountInstance2 = (string)storageAccountValue4;
                                    queueMessageInstance2.StorageAccountName = storageAccountInstance2;
                                }
                                
                                JToken queueNameValue4 = queueMessageValue4["queueName"];
                                if (queueNameValue4 != null)
                                {
                                    string queueNameInstance2 = (string)queueNameValue4;
                                    queueMessageInstance2.QueueName = queueNameInstance2;
                                }
                                
                                JToken sasTokenValue4 = queueMessageValue4["sasToken"];
                                if (sasTokenValue4 != null)
                                {
                                    string sasTokenInstance2 = (string)sasTokenValue4;
                                    queueMessageInstance2.SasToken = sasTokenInstance2;
                                }
                                
                                JToken messageValue4 = queueMessageValue4["message"];
                                if (messageValue4 != null)
                                {
                                    string messageInstance2 = (string)messageValue4;
                                    queueMessageInstance2.Message = messageInstance2;
                                }
                            }
                        }
                        
                        JToken recurrenceValue2 = responseDoc["recurrence"];
                        if (recurrenceValue2 != null)
                        {
                            JobRecurrence recurrenceInstance = new JobRecurrence();
                            jobInstance.Recurrence = recurrenceInstance;
                            
                            JToken frequencyValue2 = recurrenceValue2["frequency"];
                            if (frequencyValue2 != null)
                            {
                                JobRecurrenceFrequency frequencyInstance = SchedulerClient.ParseJobRecurrenceFrequency((string)frequencyValue2);
                                recurrenceInstance.Frequency = frequencyInstance;
                            }
                            
                            JToken intervalValue2 = recurrenceValue2["interval"];
                            if (intervalValue2 != null)
                            {
                                int intervalInstance = (int)intervalValue2;
                                recurrenceInstance.Interval = intervalInstance;
                            }
                            
                            JToken countValue2 = recurrenceValue2["count"];
                            if (countValue2 != null)
                            {
                                int countInstance = (int)countValue2;
                                recurrenceInstance.Count = countInstance;
                            }
                            
                            JToken endTimeValue2 = recurrenceValue2["endTime"];
                            if (endTimeValue2 != null)
                            {
                                DateTime endTimeInstance = (DateTime)endTimeValue2;
                                recurrenceInstance.EndTime = endTimeInstance;
                            }
                            
                            JToken scheduleValue2 = recurrenceValue2["schedule"];
                            if (scheduleValue2 != null)
                            {
                                JobRecurrenceSchedule scheduleInstance = new JobRecurrenceSchedule();
                                recurrenceInstance.Schedule = scheduleInstance;
                                
                                JArray minutesArray2 = (JArray)scheduleValue2["minutes"];
                                if (minutesArray2 != null)
                                {
                                    foreach (JToken minutesValue in minutesArray2)
                                    {
                                        scheduleInstance.Minutes.Add((int)minutesValue);
                                    }
                                }
                                
                                JArray hoursArray2 = (JArray)scheduleValue2["hours"];
                                if (hoursArray2 != null)
                                {
                                    foreach (JToken hoursValue in hoursArray2)
                                    {
                                        scheduleInstance.Hours.Add((int)hoursValue);
                                    }
                                }
                                
                                JArray daysArray2 = (JArray)scheduleValue2["days"];
                                if (daysArray2 != null)
                                {
                                    foreach (JToken daysValue in daysArray2)
                                    {
                                        scheduleInstance.Days.Add(SchedulerClient.ParseJobScheduleDay((string)daysValue));
                                    }
                                }
                                
                                JArray monthsArray2 = (JArray)scheduleValue2["months"];
                                if (monthsArray2 != null)
                                {
                                    foreach (JToken monthsValue in monthsArray2)
                                    {
                                        scheduleInstance.Months.Add((int)monthsValue);
                                    }
                                }
                                
                                JArray monthDaysArray2 = (JArray)scheduleValue2["monthDays"];
                                if (monthDaysArray2 != null)
                                {
                                    foreach (JToken monthDaysValue in monthDaysArray2)
                                    {
                                        scheduleInstance.MonthDays.Add((int)monthDaysValue);
                                    }
                                }
                                
                                JArray monthlyOccurrencesArray2 = (JArray)scheduleValue2["monthlyOccurrences"];
                                if (monthlyOccurrencesArray2 != null)
                                {
                                    foreach (JToken monthlyOccurrencesValue in monthlyOccurrencesArray2)
                                    {
                                        JobScheduleMonthlyOccurrence jobScheduleMonthlyOccurrenceInstance = new JobScheduleMonthlyOccurrence();
                                        scheduleInstance.MonthlyOccurrences.Add(jobScheduleMonthlyOccurrenceInstance);
                                        
                                        JToken dayValue2 = monthlyOccurrencesValue["day"];
                                        if (dayValue2 != null)
                                        {
                                            JobScheduleDay dayInstance = SchedulerClient.ParseJobScheduleDay((string)dayValue2);
                                            jobScheduleMonthlyOccurrenceInstance.Day = dayInstance;
                                        }
                                        
                                        JToken occurrenceValue2 = monthlyOccurrencesValue["occurrence"];
                                        if (occurrenceValue2 != null)
                                        {
                                            int occurrenceInstance = (int)occurrenceValue2;
                                            jobScheduleMonthlyOccurrenceInstance.Occurrence = occurrenceInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        JToken statusValue = responseDoc["status"];
                        if (statusValue != null)
                        {
                            JobStatus statusInstance = new JobStatus();
                            jobInstance.Status = statusInstance;
                            
                            JToken lastExecutionTimeValue = statusValue["lastExecutionTime"];
                            if (lastExecutionTimeValue != null)
                            {
                                DateTime lastExecutionTimeInstance = (DateTime)lastExecutionTimeValue;
                                statusInstance.LastExecutionTime = lastExecutionTimeInstance;
                            }
                            
                            JToken nextExecutionTimeValue = statusValue["nextExecutionTime"];
                            if (nextExecutionTimeValue != null)
                            {
                                DateTime nextExecutionTimeInstance = (DateTime)nextExecutionTimeValue;
                                statusInstance.NextExecutionTime = nextExecutionTimeInstance;
                            }
                            
                            JToken executionCountValue = statusValue["executionCount"];
                            if (executionCountValue != null)
                            {
                                int executionCountInstance = (int)executionCountValue;
                                statusInstance.ExecutionCount = executionCountInstance;
                            }
                            
                            JToken failureCountValue = statusValue["failureCount"];
                            if (failureCountValue != null)
                            {
                                int failureCountInstance = (int)failureCountValue;
                                statusInstance.FailureCount = failureCountInstance;
                            }
                            
                            JToken faultedCountValue = statusValue["faultedCount"];
                            if (faultedCountValue != null)
                            {
                                int faultedCountInstance = (int)faultedCountValue;
                                statusInstance.FaultedCount = faultedCountInstance;
                            }
                        }
                        
                        JToken stateValue = responseDoc["state"];
                        if (stateValue != null)
                        {
                            JobState stateInstance = SchedulerClient.ParseJobState((string)stateValue);
                            jobInstance.State = stateInstance;
                        }
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Deletes a job.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to delete.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<OperationResponse> DeleteAsync(string jobId, CancellationToken cancellationToken)
        {
            // Validate
            if (jobId == null)
            {
                throw new ArgumentNullException("jobId");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                Tracing.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs/" + jobId + "?api-version=2013-10-31_Preview";
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromXml(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The status of an individual job can be access by a GET call to a
        /// job's address, jobId.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to get.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Job operation response.
        /// </returns>
        public async Task<JobGetResponse> GetAsync(string jobId, CancellationToken cancellationToken)
        {
            // Validate
            if (jobId == null)
            {
                throw new ArgumentNullException("jobId");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                Tracing.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs/" + jobId + "?api-version=2013-10-31_Preview";
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobGetResponse result = new JobGetResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);
                    
                    if (responseDoc != null)
                    {
                        Job jobInstance = new Job();
                        result.Job = jobInstance;
                        
                        JToken idValue = responseDoc["id"];
                        if (idValue != null)
                        {
                            string idInstance = (string)idValue;
                            jobInstance.Id = idInstance;
                        }
                        
                        JToken startTimeValue = responseDoc["startTime"];
                        if (startTimeValue != null)
                        {
                            DateTime startTimeInstance = (DateTime)startTimeValue;
                            jobInstance.StartTime = startTimeInstance;
                        }
                        
                        JToken actionValue = responseDoc["action"];
                        if (actionValue != null)
                        {
                            JobAction actionInstance = new JobAction();
                            jobInstance.Action = actionInstance;
                            
                            JToken typeValue = actionValue["type"];
                            if (typeValue != null)
                            {
                                JobActionType typeInstance = SchedulerClient.ParseJobActionType((string)typeValue);
                                actionInstance.Type = typeInstance;
                            }
                            
                            JToken retryPolicyValue = actionValue["retryPolicy"];
                            if (retryPolicyValue != null)
                            {
                                RetryPolicy retryPolicyInstance = new RetryPolicy();
                                actionInstance.RetryPolicy = retryPolicyInstance;
                                
                                JToken retryTypeValue = retryPolicyValue["retryType"];
                                if (retryTypeValue != null)
                                {
                                    RetryType retryTypeInstance = SchedulerClient.ParseRetryType((string)retryTypeValue);
                                    retryPolicyInstance.RetryType = retryTypeInstance;
                                }
                                
                                JToken retryIntervalValue = retryPolicyValue["retryInterval"];
                                if (retryIntervalValue != null)
                                {
                                    TimeSpan retryIntervalInstance = TimeSpan.Parse((string)retryIntervalValue, CultureInfo.InvariantCulture);
                                    retryPolicyInstance.RetryInterval = retryIntervalInstance;
                                }
                                
                                JToken retryCountValue = retryPolicyValue["retryCount"];
                                if (retryCountValue != null)
                                {
                                    int retryCountInstance = (int)retryCountValue;
                                    retryPolicyInstance.RetryCount = retryCountInstance;
                                }
                            }
                            
                            JToken errorActionValue = actionValue["errorAction"];
                            if (errorActionValue != null)
                            {
                                JobErrorAction errorActionInstance = new JobErrorAction();
                                actionInstance.ErrorAction = errorActionInstance;
                                
                                JToken typeValue2 = errorActionValue["type"];
                                if (typeValue2 != null)
                                {
                                    JobActionType typeInstance2 = SchedulerClient.ParseJobActionType((string)typeValue2);
                                    errorActionInstance.Type = typeInstance2;
                                }
                                
                                JToken requestValue = errorActionValue["request"];
                                if (requestValue != null)
                                {
                                    JobHttpRequest requestInstance = new JobHttpRequest();
                                    errorActionInstance.Request = requestInstance;
                                    
                                    JToken uriValue = requestValue["uri"];
                                    if (uriValue != null)
                                    {
                                        Uri uriInstance = TypeConversion.TryParseUri((string)uriValue);
                                        requestInstance.Uri = uriInstance;
                                    }
                                    
                                    JToken methodValue = requestValue["method"];
                                    if (methodValue != null)
                                    {
                                        string methodInstance = (string)methodValue;
                                        requestInstance.Method = methodInstance;
                                    }
                                    
                                    JToken headersSequenceElement = requestValue["headers"];
                                    if (headersSequenceElement != null)
                                    {
                                        foreach (JProperty property in headersSequenceElement)
                                        {
                                            string headersKey = (string)property.Name;
                                            string headersValue = (string)property.Value;
                                            requestInstance.Headers.Add(headersKey, headersValue);
                                        }
                                    }
                                    
                                    JToken bodyValue = requestValue["body"];
                                    if (bodyValue != null)
                                    {
                                        string bodyInstance = (string)bodyValue;
                                        requestInstance.Body = bodyInstance;
                                    }
                                }
                                
                                JToken queueMessageValue = errorActionValue["queueMessage"];
                                if (queueMessageValue != null)
                                {
                                    JobQueueMessage queueMessageInstance = new JobQueueMessage();
                                    errorActionInstance.QueueMessage = queueMessageInstance;
                                    
                                    JToken storageAccountValue = queueMessageValue["storageAccount"];
                                    if (storageAccountValue != null)
                                    {
                                        string storageAccountInstance = (string)storageAccountValue;
                                        queueMessageInstance.StorageAccountName = storageAccountInstance;
                                    }
                                    
                                    JToken queueNameValue = queueMessageValue["queueName"];
                                    if (queueNameValue != null)
                                    {
                                        string queueNameInstance = (string)queueNameValue;
                                        queueMessageInstance.QueueName = queueNameInstance;
                                    }
                                    
                                    JToken sasTokenValue = queueMessageValue["sasToken"];
                                    if (sasTokenValue != null)
                                    {
                                        string sasTokenInstance = (string)sasTokenValue;
                                        queueMessageInstance.SasToken = sasTokenInstance;
                                    }
                                    
                                    JToken messageValue = queueMessageValue["message"];
                                    if (messageValue != null)
                                    {
                                        string messageInstance = (string)messageValue;
                                        queueMessageInstance.Message = messageInstance;
                                    }
                                }
                            }
                            
                            JToken requestValue2 = actionValue["request"];
                            if (requestValue2 != null)
                            {
                                JobHttpRequest requestInstance2 = new JobHttpRequest();
                                actionInstance.Request = requestInstance2;
                                
                                JToken uriValue2 = requestValue2["uri"];
                                if (uriValue2 != null)
                                {
                                    Uri uriInstance2 = TypeConversion.TryParseUri((string)uriValue2);
                                    requestInstance2.Uri = uriInstance2;
                                }
                                
                                JToken methodValue2 = requestValue2["method"];
                                if (methodValue2 != null)
                                {
                                    string methodInstance2 = (string)methodValue2;
                                    requestInstance2.Method = methodInstance2;
                                }
                                
                                JToken headersSequenceElement2 = requestValue2["headers"];
                                if (headersSequenceElement2 != null)
                                {
                                    foreach (JProperty property2 in headersSequenceElement2)
                                    {
                                        string headersKey2 = (string)property2.Name;
                                        string headersValue2 = (string)property2.Value;
                                        requestInstance2.Headers.Add(headersKey2, headersValue2);
                                    }
                                }
                                
                                JToken bodyValue2 = requestValue2["body"];
                                if (bodyValue2 != null)
                                {
                                    string bodyInstance2 = (string)bodyValue2;
                                    requestInstance2.Body = bodyInstance2;
                                }
                            }
                            
                            JToken queueMessageValue2 = actionValue["queueMessage"];
                            if (queueMessageValue2 != null)
                            {
                                JobQueueMessage queueMessageInstance2 = new JobQueueMessage();
                                actionInstance.QueueMessage = queueMessageInstance2;
                                
                                JToken storageAccountValue2 = queueMessageValue2["storageAccount"];
                                if (storageAccountValue2 != null)
                                {
                                    string storageAccountInstance2 = (string)storageAccountValue2;
                                    queueMessageInstance2.StorageAccountName = storageAccountInstance2;
                                }
                                
                                JToken queueNameValue2 = queueMessageValue2["queueName"];
                                if (queueNameValue2 != null)
                                {
                                    string queueNameInstance2 = (string)queueNameValue2;
                                    queueMessageInstance2.QueueName = queueNameInstance2;
                                }
                                
                                JToken sasTokenValue2 = queueMessageValue2["sasToken"];
                                if (sasTokenValue2 != null)
                                {
                                    string sasTokenInstance2 = (string)sasTokenValue2;
                                    queueMessageInstance2.SasToken = sasTokenInstance2;
                                }
                                
                                JToken messageValue2 = queueMessageValue2["message"];
                                if (messageValue2 != null)
                                {
                                    string messageInstance2 = (string)messageValue2;
                                    queueMessageInstance2.Message = messageInstance2;
                                }
                            }
                        }
                        
                        JToken recurrenceValue = responseDoc["recurrence"];
                        if (recurrenceValue != null)
                        {
                            JobRecurrence recurrenceInstance = new JobRecurrence();
                            jobInstance.Recurrence = recurrenceInstance;
                            
                            JToken frequencyValue = recurrenceValue["frequency"];
                            if (frequencyValue != null)
                            {
                                JobRecurrenceFrequency frequencyInstance = SchedulerClient.ParseJobRecurrenceFrequency((string)frequencyValue);
                                recurrenceInstance.Frequency = frequencyInstance;
                            }
                            
                            JToken intervalValue = recurrenceValue["interval"];
                            if (intervalValue != null)
                            {
                                int intervalInstance = (int)intervalValue;
                                recurrenceInstance.Interval = intervalInstance;
                            }
                            
                            JToken countValue = recurrenceValue["count"];
                            if (countValue != null)
                            {
                                int countInstance = (int)countValue;
                                recurrenceInstance.Count = countInstance;
                            }
                            
                            JToken endTimeValue = recurrenceValue["endTime"];
                            if (endTimeValue != null)
                            {
                                DateTime endTimeInstance = (DateTime)endTimeValue;
                                recurrenceInstance.EndTime = endTimeInstance;
                            }
                            
                            JToken scheduleValue = recurrenceValue["schedule"];
                            if (scheduleValue != null)
                            {
                                JobRecurrenceSchedule scheduleInstance = new JobRecurrenceSchedule();
                                recurrenceInstance.Schedule = scheduleInstance;
                                
                                JArray minutesArray = (JArray)scheduleValue["minutes"];
                                if (minutesArray != null)
                                {
                                    foreach (JToken minutesValue in minutesArray)
                                    {
                                        scheduleInstance.Minutes.Add((int)minutesValue);
                                    }
                                }
                                
                                JArray hoursArray = (JArray)scheduleValue["hours"];
                                if (hoursArray != null)
                                {
                                    foreach (JToken hoursValue in hoursArray)
                                    {
                                        scheduleInstance.Hours.Add((int)hoursValue);
                                    }
                                }
                                
                                JArray daysArray = (JArray)scheduleValue["days"];
                                if (daysArray != null)
                                {
                                    foreach (JToken daysValue in daysArray)
                                    {
                                        scheduleInstance.Days.Add(SchedulerClient.ParseJobScheduleDay((string)daysValue));
                                    }
                                }
                                
                                JArray monthsArray = (JArray)scheduleValue["months"];
                                if (monthsArray != null)
                                {
                                    foreach (JToken monthsValue in monthsArray)
                                    {
                                        scheduleInstance.Months.Add((int)monthsValue);
                                    }
                                }
                                
                                JArray monthDaysArray = (JArray)scheduleValue["monthDays"];
                                if (monthDaysArray != null)
                                {
                                    foreach (JToken monthDaysValue in monthDaysArray)
                                    {
                                        scheduleInstance.MonthDays.Add((int)monthDaysValue);
                                    }
                                }
                                
                                JArray monthlyOccurrencesArray = (JArray)scheduleValue["monthlyOccurrences"];
                                if (monthlyOccurrencesArray != null)
                                {
                                    foreach (JToken monthlyOccurrencesValue in monthlyOccurrencesArray)
                                    {
                                        JobScheduleMonthlyOccurrence jobScheduleMonthlyOccurrenceInstance = new JobScheduleMonthlyOccurrence();
                                        scheduleInstance.MonthlyOccurrences.Add(jobScheduleMonthlyOccurrenceInstance);
                                        
                                        JToken dayValue = monthlyOccurrencesValue["day"];
                                        if (dayValue != null)
                                        {
                                            JobScheduleDay dayInstance = SchedulerClient.ParseJobScheduleDay((string)dayValue);
                                            jobScheduleMonthlyOccurrenceInstance.Day = dayInstance;
                                        }
                                        
                                        JToken occurrenceValue = monthlyOccurrencesValue["occurrence"];
                                        if (occurrenceValue != null)
                                        {
                                            int occurrenceInstance = (int)occurrenceValue;
                                            jobScheduleMonthlyOccurrenceInstance.Occurrence = occurrenceInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        JToken statusValue = responseDoc["status"];
                        if (statusValue != null)
                        {
                            JobStatus statusInstance = new JobStatus();
                            jobInstance.Status = statusInstance;
                            
                            JToken lastExecutionTimeValue = statusValue["lastExecutionTime"];
                            if (lastExecutionTimeValue != null)
                            {
                                DateTime lastExecutionTimeInstance = (DateTime)lastExecutionTimeValue;
                                statusInstance.LastExecutionTime = lastExecutionTimeInstance;
                            }
                            
                            JToken nextExecutionTimeValue = statusValue["nextExecutionTime"];
                            if (nextExecutionTimeValue != null)
                            {
                                DateTime nextExecutionTimeInstance = (DateTime)nextExecutionTimeValue;
                                statusInstance.NextExecutionTime = nextExecutionTimeInstance;
                            }
                            
                            JToken executionCountValue = statusValue["executionCount"];
                            if (executionCountValue != null)
                            {
                                int executionCountInstance = (int)executionCountValue;
                                statusInstance.ExecutionCount = executionCountInstance;
                            }
                            
                            JToken failureCountValue = statusValue["failureCount"];
                            if (failureCountValue != null)
                            {
                                int failureCountInstance = (int)failureCountValue;
                                statusInstance.FailureCount = failureCountInstance;
                            }
                            
                            JToken faultedCountValue = statusValue["faultedCount"];
                            if (faultedCountValue != null)
                            {
                                int faultedCountInstance = (int)faultedCountValue;
                                statusInstance.FaultedCount = faultedCountInstance;
                            }
                        }
                        
                        JToken stateValue = responseDoc["state"];
                        if (stateValue != null)
                        {
                            JobState stateInstance = SchedulerClient.ParseJobState((string)stateValue);
                            jobInstance.State = stateInstance;
                        }
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Job history tracks the updates and executions of a job.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to get the history of.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Get Job History operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Get Job History operation response.
        /// </returns>
        public async Task<JobGetHistoryResponse> GetHistoryAsync(string jobId, JobGetHistoryParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (jobId == null)
            {
                throw new ArgumentNullException("jobId");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "GetHistoryAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs/" + jobId + "/history?api-version=2013-10-31_Preview";
            if (parameters.State != null)
            {
                url = url + "&state=" + Uri.EscapeUriString(SchedulerClient.JobStateToString(parameters.State.Value));
            }
            if (parameters.Skip != null)
            {
                url = url + "&$skip=" + Uri.EscapeUriString(parameters.Skip.Value.ToString());
            }
            if (parameters.Top != null)
            {
                url = url + "&$top=" + Uri.EscapeUriString(parameters.Top.Value.ToString());
            }
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobGetHistoryResponse result = new JobGetHistoryResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);
                    
                    JArray jobHistoryArray = (JArray)responseDoc;
                    if (jobHistoryArray != null)
                    {
                        foreach (JToken jobHistoryValue in jobHistoryArray)
                        {
                            JobGetHistoryResponse.JobHistoryEntry jobHistoryEntryInstance = new JobGetHistoryResponse.JobHistoryEntry();
                            result.JobHistory.Add(jobHistoryEntryInstance);
                            
                            JToken jobIdValue = jobHistoryValue["jobId"];
                            if (jobIdValue != null)
                            {
                                string jobIdInstance = (string)jobIdValue;
                                jobHistoryEntryInstance.Id = jobIdInstance;
                            }
                            
                            JToken recordNumberValue = jobHistoryValue["recordNumber"];
                            if (recordNumberValue != null)
                            {
                                int recordNumberInstance = (int)recordNumberValue;
                                jobHistoryEntryInstance.RecordNumber = recordNumberInstance;
                            }
                            
                            JToken timestampValue = jobHistoryValue["timestamp"];
                            if (timestampValue != null)
                            {
                                DateTime timestampInstance = (DateTime)timestampValue;
                                jobHistoryEntryInstance.Timestamp = timestampInstance;
                            }
                            
                            JToken startTimeValue = jobHistoryValue["startTime"];
                            if (startTimeValue != null)
                            {
                                DateTime startTimeInstance = (DateTime)startTimeValue;
                                jobHistoryEntryInstance.StartTime = startTimeInstance;
                            }
                            
                            JToken endTimeValue = jobHistoryValue["endTime"];
                            if (endTimeValue != null)
                            {
                                DateTime endTimeInstance = (DateTime)endTimeValue;
                                jobHistoryEntryInstance.EndTime = endTimeInstance;
                            }
                            
                            JToken stateValue = jobHistoryValue["state"];
                            if (stateValue != null)
                            {
                                JobState stateInstance = SchedulerClient.ParseJobState((string)stateValue);
                                jobHistoryEntryInstance.State = stateInstance;
                            }
                            
                            JToken messageValue = jobHistoryValue["message"];
                            if (messageValue != null)
                            {
                                string messageInstance = (string)messageValue;
                                jobHistoryEntryInstance.Message = messageInstance;
                            }
                        }
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Fetch all jobs in a job collection.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the List Jobs operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List Jobs operation response.
        /// </returns>
        public async Task<JobListResponse> ListAsync(JobListParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs?api-version=2013-10-31_Preview";
            if (parameters.State != null)
            {
                url = url + "&state=" + Uri.EscapeUriString(SchedulerClient.JobStateToString(parameters.State.Value));
            }
            if (parameters.Skip != null)
            {
                url = url + "&$skip=" + Uri.EscapeUriString(parameters.Skip.Value.ToString());
            }
            if (parameters.Top != null)
            {
                url = url + "&$top=" + Uri.EscapeUriString(parameters.Top.Value.ToString());
            }
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobListResponse result = new JobListResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);
                    
                    JArray jobsArray = (JArray)responseDoc;
                    if (jobsArray != null)
                    {
                        foreach (JToken jobsValue in jobsArray)
                        {
                            Job jobInstance = new Job();
                            result.Jobs.Add(jobInstance);
                            
                            JToken idValue = jobsValue["id"];
                            if (idValue != null)
                            {
                                string idInstance = (string)idValue;
                                jobInstance.Id = idInstance;
                            }
                            
                            JToken startTimeValue = jobsValue["startTime"];
                            if (startTimeValue != null)
                            {
                                DateTime startTimeInstance = (DateTime)startTimeValue;
                                jobInstance.StartTime = startTimeInstance;
                            }
                            
                            JToken actionValue = jobsValue["action"];
                            if (actionValue != null)
                            {
                                JobAction actionInstance = new JobAction();
                                jobInstance.Action = actionInstance;
                                
                                JToken typeValue = actionValue["type"];
                                if (typeValue != null)
                                {
                                    JobActionType typeInstance = SchedulerClient.ParseJobActionType((string)typeValue);
                                    actionInstance.Type = typeInstance;
                                }
                                
                                JToken retryPolicyValue = actionValue["retryPolicy"];
                                if (retryPolicyValue != null)
                                {
                                    RetryPolicy retryPolicyInstance = new RetryPolicy();
                                    actionInstance.RetryPolicy = retryPolicyInstance;
                                    
                                    JToken retryTypeValue = retryPolicyValue["retryType"];
                                    if (retryTypeValue != null)
                                    {
                                        RetryType retryTypeInstance = SchedulerClient.ParseRetryType((string)retryTypeValue);
                                        retryPolicyInstance.RetryType = retryTypeInstance;
                                    }
                                    
                                    JToken retryIntervalValue = retryPolicyValue["retryInterval"];
                                    if (retryIntervalValue != null)
                                    {
                                        TimeSpan retryIntervalInstance = TimeSpan.Parse((string)retryIntervalValue, CultureInfo.InvariantCulture);
                                        retryPolicyInstance.RetryInterval = retryIntervalInstance;
                                    }
                                    
                                    JToken retryCountValue = retryPolicyValue["retryCount"];
                                    if (retryCountValue != null)
                                    {
                                        int retryCountInstance = (int)retryCountValue;
                                        retryPolicyInstance.RetryCount = retryCountInstance;
                                    }
                                }
                                
                                JToken errorActionValue = actionValue["errorAction"];
                                if (errorActionValue != null)
                                {
                                    JobErrorAction errorActionInstance = new JobErrorAction();
                                    actionInstance.ErrorAction = errorActionInstance;
                                    
                                    JToken typeValue2 = errorActionValue["type"];
                                    if (typeValue2 != null)
                                    {
                                        JobActionType typeInstance2 = SchedulerClient.ParseJobActionType((string)typeValue2);
                                        errorActionInstance.Type = typeInstance2;
                                    }
                                    
                                    JToken requestValue = errorActionValue["request"];
                                    if (requestValue != null)
                                    {
                                        JobHttpRequest requestInstance = new JobHttpRequest();
                                        errorActionInstance.Request = requestInstance;
                                        
                                        JToken uriValue = requestValue["uri"];
                                        if (uriValue != null)
                                        {
                                            Uri uriInstance = TypeConversion.TryParseUri((string)uriValue);
                                            requestInstance.Uri = uriInstance;
                                        }
                                        
                                        JToken methodValue = requestValue["method"];
                                        if (methodValue != null)
                                        {
                                            string methodInstance = (string)methodValue;
                                            requestInstance.Method = methodInstance;
                                        }
                                        
                                        JToken headersSequenceElement = requestValue["headers"];
                                        if (headersSequenceElement != null)
                                        {
                                            foreach (JProperty property in headersSequenceElement)
                                            {
                                                string headersKey = (string)property.Name;
                                                string headersValue = (string)property.Value;
                                                requestInstance.Headers.Add(headersKey, headersValue);
                                            }
                                        }
                                        
                                        JToken bodyValue = requestValue["body"];
                                        if (bodyValue != null)
                                        {
                                            string bodyInstance = (string)bodyValue;
                                            requestInstance.Body = bodyInstance;
                                        }
                                    }
                                    
                                    JToken queueMessageValue = errorActionValue["queueMessage"];
                                    if (queueMessageValue != null)
                                    {
                                        JobQueueMessage queueMessageInstance = new JobQueueMessage();
                                        errorActionInstance.QueueMessage = queueMessageInstance;
                                        
                                        JToken storageAccountValue = queueMessageValue["storageAccount"];
                                        if (storageAccountValue != null)
                                        {
                                            string storageAccountInstance = (string)storageAccountValue;
                                            queueMessageInstance.StorageAccountName = storageAccountInstance;
                                        }
                                        
                                        JToken queueNameValue = queueMessageValue["queueName"];
                                        if (queueNameValue != null)
                                        {
                                            string queueNameInstance = (string)queueNameValue;
                                            queueMessageInstance.QueueName = queueNameInstance;
                                        }
                                        
                                        JToken sasTokenValue = queueMessageValue["sasToken"];
                                        if (sasTokenValue != null)
                                        {
                                            string sasTokenInstance = (string)sasTokenValue;
                                            queueMessageInstance.SasToken = sasTokenInstance;
                                        }
                                        
                                        JToken messageValue = queueMessageValue["message"];
                                        if (messageValue != null)
                                        {
                                            string messageInstance = (string)messageValue;
                                            queueMessageInstance.Message = messageInstance;
                                        }
                                    }
                                }
                                
                                JToken requestValue2 = actionValue["request"];
                                if (requestValue2 != null)
                                {
                                    JobHttpRequest requestInstance2 = new JobHttpRequest();
                                    actionInstance.Request = requestInstance2;
                                    
                                    JToken uriValue2 = requestValue2["uri"];
                                    if (uriValue2 != null)
                                    {
                                        Uri uriInstance2 = TypeConversion.TryParseUri((string)uriValue2);
                                        requestInstance2.Uri = uriInstance2;
                                    }
                                    
                                    JToken methodValue2 = requestValue2["method"];
                                    if (methodValue2 != null)
                                    {
                                        string methodInstance2 = (string)methodValue2;
                                        requestInstance2.Method = methodInstance2;
                                    }
                                    
                                    JToken headersSequenceElement2 = requestValue2["headers"];
                                    if (headersSequenceElement2 != null)
                                    {
                                        foreach (JProperty property2 in headersSequenceElement2)
                                        {
                                            string headersKey2 = (string)property2.Name;
                                            string headersValue2 = (string)property2.Value;
                                            requestInstance2.Headers.Add(headersKey2, headersValue2);
                                        }
                                    }
                                    
                                    JToken bodyValue2 = requestValue2["body"];
                                    if (bodyValue2 != null)
                                    {
                                        string bodyInstance2 = (string)bodyValue2;
                                        requestInstance2.Body = bodyInstance2;
                                    }
                                }
                                
                                JToken queueMessageValue2 = actionValue["queueMessage"];
                                if (queueMessageValue2 != null)
                                {
                                    JobQueueMessage queueMessageInstance2 = new JobQueueMessage();
                                    actionInstance.QueueMessage = queueMessageInstance2;
                                    
                                    JToken storageAccountValue2 = queueMessageValue2["storageAccount"];
                                    if (storageAccountValue2 != null)
                                    {
                                        string storageAccountInstance2 = (string)storageAccountValue2;
                                        queueMessageInstance2.StorageAccountName = storageAccountInstance2;
                                    }
                                    
                                    JToken queueNameValue2 = queueMessageValue2["queueName"];
                                    if (queueNameValue2 != null)
                                    {
                                        string queueNameInstance2 = (string)queueNameValue2;
                                        queueMessageInstance2.QueueName = queueNameInstance2;
                                    }
                                    
                                    JToken sasTokenValue2 = queueMessageValue2["sasToken"];
                                    if (sasTokenValue2 != null)
                                    {
                                        string sasTokenInstance2 = (string)sasTokenValue2;
                                        queueMessageInstance2.SasToken = sasTokenInstance2;
                                    }
                                    
                                    JToken messageValue2 = queueMessageValue2["message"];
                                    if (messageValue2 != null)
                                    {
                                        string messageInstance2 = (string)messageValue2;
                                        queueMessageInstance2.Message = messageInstance2;
                                    }
                                }
                            }
                            
                            JToken recurrenceValue = jobsValue["recurrence"];
                            if (recurrenceValue != null)
                            {
                                JobRecurrence recurrenceInstance = new JobRecurrence();
                                jobInstance.Recurrence = recurrenceInstance;
                                
                                JToken frequencyValue = recurrenceValue["frequency"];
                                if (frequencyValue != null)
                                {
                                    JobRecurrenceFrequency frequencyInstance = SchedulerClient.ParseJobRecurrenceFrequency((string)frequencyValue);
                                    recurrenceInstance.Frequency = frequencyInstance;
                                }
                                
                                JToken intervalValue = recurrenceValue["interval"];
                                if (intervalValue != null)
                                {
                                    int intervalInstance = (int)intervalValue;
                                    recurrenceInstance.Interval = intervalInstance;
                                }
                                
                                JToken countValue = recurrenceValue["count"];
                                if (countValue != null)
                                {
                                    int countInstance = (int)countValue;
                                    recurrenceInstance.Count = countInstance;
                                }
                                
                                JToken endTimeValue = recurrenceValue["endTime"];
                                if (endTimeValue != null)
                                {
                                    DateTime endTimeInstance = (DateTime)endTimeValue;
                                    recurrenceInstance.EndTime = endTimeInstance;
                                }
                                
                                JToken scheduleValue = recurrenceValue["schedule"];
                                if (scheduleValue != null)
                                {
                                    JobRecurrenceSchedule scheduleInstance = new JobRecurrenceSchedule();
                                    recurrenceInstance.Schedule = scheduleInstance;
                                    
                                    JArray minutesArray = (JArray)scheduleValue["minutes"];
                                    if (minutesArray != null)
                                    {
                                        foreach (JToken minutesValue in minutesArray)
                                        {
                                            scheduleInstance.Minutes.Add((int)minutesValue);
                                        }
                                    }
                                    
                                    JArray hoursArray = (JArray)scheduleValue["hours"];
                                    if (hoursArray != null)
                                    {
                                        foreach (JToken hoursValue in hoursArray)
                                        {
                                            scheduleInstance.Hours.Add((int)hoursValue);
                                        }
                                    }
                                    
                                    JArray daysArray = (JArray)scheduleValue["days"];
                                    if (daysArray != null)
                                    {
                                        foreach (JToken daysValue in daysArray)
                                        {
                                            scheduleInstance.Days.Add(SchedulerClient.ParseJobScheduleDay((string)daysValue));
                                        }
                                    }
                                    
                                    JArray monthsArray = (JArray)scheduleValue["months"];
                                    if (monthsArray != null)
                                    {
                                        foreach (JToken monthsValue in monthsArray)
                                        {
                                            scheduleInstance.Months.Add((int)monthsValue);
                                        }
                                    }
                                    
                                    JArray monthDaysArray = (JArray)scheduleValue["monthDays"];
                                    if (monthDaysArray != null)
                                    {
                                        foreach (JToken monthDaysValue in monthDaysArray)
                                        {
                                            scheduleInstance.MonthDays.Add((int)monthDaysValue);
                                        }
                                    }
                                    
                                    JArray monthlyOccurrencesArray = (JArray)scheduleValue["monthlyOccurrences"];
                                    if (monthlyOccurrencesArray != null)
                                    {
                                        foreach (JToken monthlyOccurrencesValue in monthlyOccurrencesArray)
                                        {
                                            JobScheduleMonthlyOccurrence jobScheduleMonthlyOccurrenceInstance = new JobScheduleMonthlyOccurrence();
                                            scheduleInstance.MonthlyOccurrences.Add(jobScheduleMonthlyOccurrenceInstance);
                                            
                                            JToken dayValue = monthlyOccurrencesValue["day"];
                                            if (dayValue != null)
                                            {
                                                JobScheduleDay dayInstance = SchedulerClient.ParseJobScheduleDay((string)dayValue);
                                                jobScheduleMonthlyOccurrenceInstance.Day = dayInstance;
                                            }
                                            
                                            JToken occurrenceValue = monthlyOccurrencesValue["occurrence"];
                                            if (occurrenceValue != null)
                                            {
                                                int occurrenceInstance = (int)occurrenceValue;
                                                jobScheduleMonthlyOccurrenceInstance.Occurrence = occurrenceInstance;
                                            }
                                        }
                                    }
                                }
                            }
                            
                            JToken statusValue = jobsValue["status"];
                            if (statusValue != null)
                            {
                                JobStatus statusInstance = new JobStatus();
                                jobInstance.Status = statusInstance;
                                
                                JToken lastExecutionTimeValue = statusValue["lastExecutionTime"];
                                if (lastExecutionTimeValue != null)
                                {
                                    DateTime lastExecutionTimeInstance = (DateTime)lastExecutionTimeValue;
                                    statusInstance.LastExecutionTime = lastExecutionTimeInstance;
                                }
                                
                                JToken nextExecutionTimeValue = statusValue["nextExecutionTime"];
                                if (nextExecutionTimeValue != null)
                                {
                                    DateTime nextExecutionTimeInstance = (DateTime)nextExecutionTimeValue;
                                    statusInstance.NextExecutionTime = nextExecutionTimeInstance;
                                }
                                
                                JToken executionCountValue = statusValue["executionCount"];
                                if (executionCountValue != null)
                                {
                                    int executionCountInstance = (int)executionCountValue;
                                    statusInstance.ExecutionCount = executionCountInstance;
                                }
                                
                                JToken failureCountValue = statusValue["failureCount"];
                                if (failureCountValue != null)
                                {
                                    int failureCountInstance = (int)failureCountValue;
                                    statusInstance.FailureCount = failureCountInstance;
                                }
                                
                                JToken faultedCountValue = statusValue["faultedCount"];
                                if (faultedCountValue != null)
                                {
                                    int faultedCountInstance = (int)faultedCountValue;
                                    statusInstance.FaultedCount = faultedCountInstance;
                                }
                            }
                            
                            JToken stateValue = jobsValue["state"];
                            if (stateValue != null)
                            {
                                JobState stateInstance = SchedulerClient.ParseJobState((string)stateValue);
                                jobInstance.State = stateInstance;
                            }
                        }
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Job collections can be updated through a simple PATCH operation.
        /// The format of the request is the same as that for creating a job,
        /// though if a field is unspecified we will carry forward the current
        /// value.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Jobs State operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Update Jobs State operation response.
        /// </returns>
        public async Task<JobCollectionJobsUpdateStateResponse> UpdateJobCollectionStateAsync(JobCollectionJobsUpdateStateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "UpdateJobCollectionStateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs";
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = new HttpMethod("PATCH");
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject jobCollectionJobsUpdateStateParametersValue = new JObject();
                requestDoc = jobCollectionJobsUpdateStateParametersValue;
                
                JToken stateValue = SchedulerClient.JobStateToString(parameters.State);
                jobCollectionJobsUpdateStateParametersValue["state"] = stateValue;
                
                requestContent = requestDoc.ToString(Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobCollectionJobsUpdateStateResponse result = new JobCollectionJobsUpdateStateResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);
                    
                    JArray jobsArray = (JArray)responseDoc;
                    if (jobsArray != null)
                    {
                        foreach (JToken jobsValue in jobsArray)
                        {
                            Job jobInstance = new Job();
                            result.Jobs.Add(jobInstance);
                            
                            JToken idValue = jobsValue["id"];
                            if (idValue != null)
                            {
                                string idInstance = (string)idValue;
                                jobInstance.Id = idInstance;
                            }
                            
                            JToken startTimeValue = jobsValue["startTime"];
                            if (startTimeValue != null)
                            {
                                DateTime startTimeInstance = (DateTime)startTimeValue;
                                jobInstance.StartTime = startTimeInstance;
                            }
                            
                            JToken actionValue = jobsValue["action"];
                            if (actionValue != null)
                            {
                                JobAction actionInstance = new JobAction();
                                jobInstance.Action = actionInstance;
                                
                                JToken typeValue = actionValue["type"];
                                if (typeValue != null)
                                {
                                    JobActionType typeInstance = SchedulerClient.ParseJobActionType((string)typeValue);
                                    actionInstance.Type = typeInstance;
                                }
                                
                                JToken retryPolicyValue = actionValue["retryPolicy"];
                                if (retryPolicyValue != null)
                                {
                                    RetryPolicy retryPolicyInstance = new RetryPolicy();
                                    actionInstance.RetryPolicy = retryPolicyInstance;
                                    
                                    JToken retryTypeValue = retryPolicyValue["retryType"];
                                    if (retryTypeValue != null)
                                    {
                                        RetryType retryTypeInstance = SchedulerClient.ParseRetryType((string)retryTypeValue);
                                        retryPolicyInstance.RetryType = retryTypeInstance;
                                    }
                                    
                                    JToken retryIntervalValue = retryPolicyValue["retryInterval"];
                                    if (retryIntervalValue != null)
                                    {
                                        TimeSpan retryIntervalInstance = TimeSpan.Parse((string)retryIntervalValue, CultureInfo.InvariantCulture);
                                        retryPolicyInstance.RetryInterval = retryIntervalInstance;
                                    }
                                    
                                    JToken retryCountValue = retryPolicyValue["retryCount"];
                                    if (retryCountValue != null)
                                    {
                                        int retryCountInstance = (int)retryCountValue;
                                        retryPolicyInstance.RetryCount = retryCountInstance;
                                    }
                                }
                                
                                JToken errorActionValue = actionValue["errorAction"];
                                if (errorActionValue != null)
                                {
                                    JobErrorAction errorActionInstance = new JobErrorAction();
                                    actionInstance.ErrorAction = errorActionInstance;
                                    
                                    JToken typeValue2 = errorActionValue["type"];
                                    if (typeValue2 != null)
                                    {
                                        JobActionType typeInstance2 = SchedulerClient.ParseJobActionType((string)typeValue2);
                                        errorActionInstance.Type = typeInstance2;
                                    }
                                    
                                    JToken requestValue = errorActionValue["request"];
                                    if (requestValue != null)
                                    {
                                        JobHttpRequest requestInstance = new JobHttpRequest();
                                        errorActionInstance.Request = requestInstance;
                                        
                                        JToken uriValue = requestValue["uri"];
                                        if (uriValue != null)
                                        {
                                            Uri uriInstance = TypeConversion.TryParseUri((string)uriValue);
                                            requestInstance.Uri = uriInstance;
                                        }
                                        
                                        JToken methodValue = requestValue["method"];
                                        if (methodValue != null)
                                        {
                                            string methodInstance = (string)methodValue;
                                            requestInstance.Method = methodInstance;
                                        }
                                        
                                        JToken headersSequenceElement = requestValue["headers"];
                                        if (headersSequenceElement != null)
                                        {
                                            foreach (JProperty property in headersSequenceElement)
                                            {
                                                string headersKey = (string)property.Name;
                                                string headersValue = (string)property.Value;
                                                requestInstance.Headers.Add(headersKey, headersValue);
                                            }
                                        }
                                        
                                        JToken bodyValue = requestValue["body"];
                                        if (bodyValue != null)
                                        {
                                            string bodyInstance = (string)bodyValue;
                                            requestInstance.Body = bodyInstance;
                                        }
                                    }
                                    
                                    JToken queueMessageValue = errorActionValue["queueMessage"];
                                    if (queueMessageValue != null)
                                    {
                                        JobQueueMessage queueMessageInstance = new JobQueueMessage();
                                        errorActionInstance.QueueMessage = queueMessageInstance;
                                        
                                        JToken storageAccountValue = queueMessageValue["storageAccount"];
                                        if (storageAccountValue != null)
                                        {
                                            string storageAccountInstance = (string)storageAccountValue;
                                            queueMessageInstance.StorageAccountName = storageAccountInstance;
                                        }
                                        
                                        JToken queueNameValue = queueMessageValue["queueName"];
                                        if (queueNameValue != null)
                                        {
                                            string queueNameInstance = (string)queueNameValue;
                                            queueMessageInstance.QueueName = queueNameInstance;
                                        }
                                        
                                        JToken sasTokenValue = queueMessageValue["sasToken"];
                                        if (sasTokenValue != null)
                                        {
                                            string sasTokenInstance = (string)sasTokenValue;
                                            queueMessageInstance.SasToken = sasTokenInstance;
                                        }
                                        
                                        JToken messageValue = queueMessageValue["message"];
                                        if (messageValue != null)
                                        {
                                            string messageInstance = (string)messageValue;
                                            queueMessageInstance.Message = messageInstance;
                                        }
                                    }
                                }
                                
                                JToken requestValue2 = actionValue["request"];
                                if (requestValue2 != null)
                                {
                                    JobHttpRequest requestInstance2 = new JobHttpRequest();
                                    actionInstance.Request = requestInstance2;
                                    
                                    JToken uriValue2 = requestValue2["uri"];
                                    if (uriValue2 != null)
                                    {
                                        Uri uriInstance2 = TypeConversion.TryParseUri((string)uriValue2);
                                        requestInstance2.Uri = uriInstance2;
                                    }
                                    
                                    JToken methodValue2 = requestValue2["method"];
                                    if (methodValue2 != null)
                                    {
                                        string methodInstance2 = (string)methodValue2;
                                        requestInstance2.Method = methodInstance2;
                                    }
                                    
                                    JToken headersSequenceElement2 = requestValue2["headers"];
                                    if (headersSequenceElement2 != null)
                                    {
                                        foreach (JProperty property2 in headersSequenceElement2)
                                        {
                                            string headersKey2 = (string)property2.Name;
                                            string headersValue2 = (string)property2.Value;
                                            requestInstance2.Headers.Add(headersKey2, headersValue2);
                                        }
                                    }
                                    
                                    JToken bodyValue2 = requestValue2["body"];
                                    if (bodyValue2 != null)
                                    {
                                        string bodyInstance2 = (string)bodyValue2;
                                        requestInstance2.Body = bodyInstance2;
                                    }
                                }
                                
                                JToken queueMessageValue2 = actionValue["queueMessage"];
                                if (queueMessageValue2 != null)
                                {
                                    JobQueueMessage queueMessageInstance2 = new JobQueueMessage();
                                    actionInstance.QueueMessage = queueMessageInstance2;
                                    
                                    JToken storageAccountValue2 = queueMessageValue2["storageAccount"];
                                    if (storageAccountValue2 != null)
                                    {
                                        string storageAccountInstance2 = (string)storageAccountValue2;
                                        queueMessageInstance2.StorageAccountName = storageAccountInstance2;
                                    }
                                    
                                    JToken queueNameValue2 = queueMessageValue2["queueName"];
                                    if (queueNameValue2 != null)
                                    {
                                        string queueNameInstance2 = (string)queueNameValue2;
                                        queueMessageInstance2.QueueName = queueNameInstance2;
                                    }
                                    
                                    JToken sasTokenValue2 = queueMessageValue2["sasToken"];
                                    if (sasTokenValue2 != null)
                                    {
                                        string sasTokenInstance2 = (string)sasTokenValue2;
                                        queueMessageInstance2.SasToken = sasTokenInstance2;
                                    }
                                    
                                    JToken messageValue2 = queueMessageValue2["message"];
                                    if (messageValue2 != null)
                                    {
                                        string messageInstance2 = (string)messageValue2;
                                        queueMessageInstance2.Message = messageInstance2;
                                    }
                                }
                            }
                            
                            JToken recurrenceValue = jobsValue["recurrence"];
                            if (recurrenceValue != null)
                            {
                                JobRecurrence recurrenceInstance = new JobRecurrence();
                                jobInstance.Recurrence = recurrenceInstance;
                                
                                JToken frequencyValue = recurrenceValue["frequency"];
                                if (frequencyValue != null)
                                {
                                    JobRecurrenceFrequency frequencyInstance = SchedulerClient.ParseJobRecurrenceFrequency((string)frequencyValue);
                                    recurrenceInstance.Frequency = frequencyInstance;
                                }
                                
                                JToken intervalValue = recurrenceValue["interval"];
                                if (intervalValue != null)
                                {
                                    int intervalInstance = (int)intervalValue;
                                    recurrenceInstance.Interval = intervalInstance;
                                }
                                
                                JToken countValue = recurrenceValue["count"];
                                if (countValue != null)
                                {
                                    int countInstance = (int)countValue;
                                    recurrenceInstance.Count = countInstance;
                                }
                                
                                JToken endTimeValue = recurrenceValue["endTime"];
                                if (endTimeValue != null)
                                {
                                    DateTime endTimeInstance = (DateTime)endTimeValue;
                                    recurrenceInstance.EndTime = endTimeInstance;
                                }
                                
                                JToken scheduleValue = recurrenceValue["schedule"];
                                if (scheduleValue != null)
                                {
                                    JobRecurrenceSchedule scheduleInstance = new JobRecurrenceSchedule();
                                    recurrenceInstance.Schedule = scheduleInstance;
                                    
                                    JArray minutesArray = (JArray)scheduleValue["minutes"];
                                    if (minutesArray != null)
                                    {
                                        foreach (JToken minutesValue in minutesArray)
                                        {
                                            scheduleInstance.Minutes.Add((int)minutesValue);
                                        }
                                    }
                                    
                                    JArray hoursArray = (JArray)scheduleValue["hours"];
                                    if (hoursArray != null)
                                    {
                                        foreach (JToken hoursValue in hoursArray)
                                        {
                                            scheduleInstance.Hours.Add((int)hoursValue);
                                        }
                                    }
                                    
                                    JArray daysArray = (JArray)scheduleValue["days"];
                                    if (daysArray != null)
                                    {
                                        foreach (JToken daysValue in daysArray)
                                        {
                                            scheduleInstance.Days.Add(SchedulerClient.ParseJobScheduleDay((string)daysValue));
                                        }
                                    }
                                    
                                    JArray monthsArray = (JArray)scheduleValue["months"];
                                    if (monthsArray != null)
                                    {
                                        foreach (JToken monthsValue in monthsArray)
                                        {
                                            scheduleInstance.Months.Add((int)monthsValue);
                                        }
                                    }
                                    
                                    JArray monthDaysArray = (JArray)scheduleValue["monthDays"];
                                    if (monthDaysArray != null)
                                    {
                                        foreach (JToken monthDaysValue in monthDaysArray)
                                        {
                                            scheduleInstance.MonthDays.Add((int)monthDaysValue);
                                        }
                                    }
                                    
                                    JArray monthlyOccurrencesArray = (JArray)scheduleValue["monthlyOccurrences"];
                                    if (monthlyOccurrencesArray != null)
                                    {
                                        foreach (JToken monthlyOccurrencesValue in monthlyOccurrencesArray)
                                        {
                                            JobScheduleMonthlyOccurrence jobScheduleMonthlyOccurrenceInstance = new JobScheduleMonthlyOccurrence();
                                            scheduleInstance.MonthlyOccurrences.Add(jobScheduleMonthlyOccurrenceInstance);
                                            
                                            JToken dayValue = monthlyOccurrencesValue["day"];
                                            if (dayValue != null)
                                            {
                                                JobScheduleDay dayInstance = SchedulerClient.ParseJobScheduleDay((string)dayValue);
                                                jobScheduleMonthlyOccurrenceInstance.Day = dayInstance;
                                            }
                                            
                                            JToken occurrenceValue = monthlyOccurrencesValue["occurrence"];
                                            if (occurrenceValue != null)
                                            {
                                                int occurrenceInstance = (int)occurrenceValue;
                                                jobScheduleMonthlyOccurrenceInstance.Occurrence = occurrenceInstance;
                                            }
                                        }
                                    }
                                }
                            }
                            
                            JToken statusValue = jobsValue["status"];
                            if (statusValue != null)
                            {
                                JobStatus statusInstance = new JobStatus();
                                jobInstance.Status = statusInstance;
                                
                                JToken lastExecutionTimeValue = statusValue["lastExecutionTime"];
                                if (lastExecutionTimeValue != null)
                                {
                                    DateTime lastExecutionTimeInstance = (DateTime)lastExecutionTimeValue;
                                    statusInstance.LastExecutionTime = lastExecutionTimeInstance;
                                }
                                
                                JToken nextExecutionTimeValue = statusValue["nextExecutionTime"];
                                if (nextExecutionTimeValue != null)
                                {
                                    DateTime nextExecutionTimeInstance = (DateTime)nextExecutionTimeValue;
                                    statusInstance.NextExecutionTime = nextExecutionTimeInstance;
                                }
                                
                                JToken executionCountValue = statusValue["executionCount"];
                                if (executionCountValue != null)
                                {
                                    int executionCountInstance = (int)executionCountValue;
                                    statusInstance.ExecutionCount = executionCountInstance;
                                }
                                
                                JToken failureCountValue = statusValue["failureCount"];
                                if (failureCountValue != null)
                                {
                                    int failureCountInstance = (int)failureCountValue;
                                    statusInstance.FailureCount = failureCountInstance;
                                }
                                
                                JToken faultedCountValue = statusValue["faultedCount"];
                                if (faultedCountValue != null)
                                {
                                    int faultedCountInstance = (int)faultedCountValue;
                                    statusInstance.FaultedCount = faultedCountInstance;
                                }
                            }
                            
                            JToken stateValue2 = jobsValue["state"];
                            if (stateValue2 != null)
                            {
                                JobState stateInstance = SchedulerClient.ParseJobState((string)stateValue2);
                                jobInstance.State = stateInstance;
                            }
                        }
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Jobs can be updated through a simple PATCH operation to a job's
        /// address. The format of the request is the same as that for
        /// creating a job, though if a field is unspecified we will carry
        /// forward the current value.
        /// </summary>
        /// <param name='jobId'>
        /// Id of the job to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job State operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The Update Job State operation response.
        /// </returns>
        public async Task<JobUpdateStateResponse> UpdateStateAsync(string jobId, JobUpdateStateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (jobId == null)
            {
                throw new ArgumentNullException("jobId");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "UpdateStateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = this.Client.BaseUri + this.Client.Credentials.SubscriptionId + "/cloudservices/" + this.Client.CloudServiceName + "/resources/scheduler/~/JobCollections/" + this.Client.JobCollectionName + "/jobs/" + jobId + "?api-version=2013-10-31_Preview";
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = new HttpMethod("PATCH");
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-03-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject jobUpdateStateParametersValue = new JObject();
                requestDoc = jobUpdateStateParametersValue;
                
                JToken stateValue = SchedulerClient.JobStateToString(parameters.State);
                jobUpdateStateParametersValue["state"] = stateValue;
                
                if (parameters.UpdateStateReason != null)
                {
                    JToken stateDetailsValue = parameters.UpdateStateReason;
                    jobUpdateStateParametersValue["stateDetails"] = stateDetailsValue;
                }
                
                requestContent = requestDoc.ToString(Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.CreateFromJson(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobUpdateStateResponse result = new JobUpdateStateResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    JToken responseDoc = JToken.Parse(responseContent);
                    
                    if (responseDoc != null)
                    {
                        Job jobInstance = new Job();
                        result.Job = jobInstance;
                        
                        JToken idValue = responseDoc["id"];
                        if (idValue != null)
                        {
                            string idInstance = (string)idValue;
                            jobInstance.Id = idInstance;
                        }
                        
                        JToken startTimeValue = responseDoc["startTime"];
                        if (startTimeValue != null)
                        {
                            DateTime startTimeInstance = (DateTime)startTimeValue;
                            jobInstance.StartTime = startTimeInstance;
                        }
                        
                        JToken actionValue = responseDoc["action"];
                        if (actionValue != null)
                        {
                            JobAction actionInstance = new JobAction();
                            jobInstance.Action = actionInstance;
                            
                            JToken typeValue = actionValue["type"];
                            if (typeValue != null)
                            {
                                JobActionType typeInstance = SchedulerClient.ParseJobActionType((string)typeValue);
                                actionInstance.Type = typeInstance;
                            }
                            
                            JToken retryPolicyValue = actionValue["retryPolicy"];
                            if (retryPolicyValue != null)
                            {
                                RetryPolicy retryPolicyInstance = new RetryPolicy();
                                actionInstance.RetryPolicy = retryPolicyInstance;
                                
                                JToken retryTypeValue = retryPolicyValue["retryType"];
                                if (retryTypeValue != null)
                                {
                                    RetryType retryTypeInstance = SchedulerClient.ParseRetryType((string)retryTypeValue);
                                    retryPolicyInstance.RetryType = retryTypeInstance;
                                }
                                
                                JToken retryIntervalValue = retryPolicyValue["retryInterval"];
                                if (retryIntervalValue != null)
                                {
                                    TimeSpan retryIntervalInstance = TimeSpan.Parse((string)retryIntervalValue, CultureInfo.InvariantCulture);
                                    retryPolicyInstance.RetryInterval = retryIntervalInstance;
                                }
                                
                                JToken retryCountValue = retryPolicyValue["retryCount"];
                                if (retryCountValue != null)
                                {
                                    int retryCountInstance = (int)retryCountValue;
                                    retryPolicyInstance.RetryCount = retryCountInstance;
                                }
                            }
                            
                            JToken errorActionValue = actionValue["errorAction"];
                            if (errorActionValue != null)
                            {
                                JobErrorAction errorActionInstance = new JobErrorAction();
                                actionInstance.ErrorAction = errorActionInstance;
                                
                                JToken typeValue2 = errorActionValue["type"];
                                if (typeValue2 != null)
                                {
                                    JobActionType typeInstance2 = SchedulerClient.ParseJobActionType((string)typeValue2);
                                    errorActionInstance.Type = typeInstance2;
                                }
                                
                                JToken requestValue = errorActionValue["request"];
                                if (requestValue != null)
                                {
                                    JobHttpRequest requestInstance = new JobHttpRequest();
                                    errorActionInstance.Request = requestInstance;
                                    
                                    JToken uriValue = requestValue["uri"];
                                    if (uriValue != null)
                                    {
                                        Uri uriInstance = TypeConversion.TryParseUri((string)uriValue);
                                        requestInstance.Uri = uriInstance;
                                    }
                                    
                                    JToken methodValue = requestValue["method"];
                                    if (methodValue != null)
                                    {
                                        string methodInstance = (string)methodValue;
                                        requestInstance.Method = methodInstance;
                                    }
                                    
                                    JToken headersSequenceElement = requestValue["headers"];
                                    if (headersSequenceElement != null)
                                    {
                                        foreach (JProperty property in headersSequenceElement)
                                        {
                                            string headersKey = (string)property.Name;
                                            string headersValue = (string)property.Value;
                                            requestInstance.Headers.Add(headersKey, headersValue);
                                        }
                                    }
                                    
                                    JToken bodyValue = requestValue["body"];
                                    if (bodyValue != null)
                                    {
                                        string bodyInstance = (string)bodyValue;
                                        requestInstance.Body = bodyInstance;
                                    }
                                }
                                
                                JToken queueMessageValue = errorActionValue["queueMessage"];
                                if (queueMessageValue != null)
                                {
                                    JobQueueMessage queueMessageInstance = new JobQueueMessage();
                                    errorActionInstance.QueueMessage = queueMessageInstance;
                                    
                                    JToken storageAccountValue = queueMessageValue["storageAccount"];
                                    if (storageAccountValue != null)
                                    {
                                        string storageAccountInstance = (string)storageAccountValue;
                                        queueMessageInstance.StorageAccountName = storageAccountInstance;
                                    }
                                    
                                    JToken queueNameValue = queueMessageValue["queueName"];
                                    if (queueNameValue != null)
                                    {
                                        string queueNameInstance = (string)queueNameValue;
                                        queueMessageInstance.QueueName = queueNameInstance;
                                    }
                                    
                                    JToken sasTokenValue = queueMessageValue["sasToken"];
                                    if (sasTokenValue != null)
                                    {
                                        string sasTokenInstance = (string)sasTokenValue;
                                        queueMessageInstance.SasToken = sasTokenInstance;
                                    }
                                    
                                    JToken messageValue = queueMessageValue["message"];
                                    if (messageValue != null)
                                    {
                                        string messageInstance = (string)messageValue;
                                        queueMessageInstance.Message = messageInstance;
                                    }
                                }
                            }
                            
                            JToken requestValue2 = actionValue["request"];
                            if (requestValue2 != null)
                            {
                                JobHttpRequest requestInstance2 = new JobHttpRequest();
                                actionInstance.Request = requestInstance2;
                                
                                JToken uriValue2 = requestValue2["uri"];
                                if (uriValue2 != null)
                                {
                                    Uri uriInstance2 = TypeConversion.TryParseUri((string)uriValue2);
                                    requestInstance2.Uri = uriInstance2;
                                }
                                
                                JToken methodValue2 = requestValue2["method"];
                                if (methodValue2 != null)
                                {
                                    string methodInstance2 = (string)methodValue2;
                                    requestInstance2.Method = methodInstance2;
                                }
                                
                                JToken headersSequenceElement2 = requestValue2["headers"];
                                if (headersSequenceElement2 != null)
                                {
                                    foreach (JProperty property2 in headersSequenceElement2)
                                    {
                                        string headersKey2 = (string)property2.Name;
                                        string headersValue2 = (string)property2.Value;
                                        requestInstance2.Headers.Add(headersKey2, headersValue2);
                                    }
                                }
                                
                                JToken bodyValue2 = requestValue2["body"];
                                if (bodyValue2 != null)
                                {
                                    string bodyInstance2 = (string)bodyValue2;
                                    requestInstance2.Body = bodyInstance2;
                                }
                            }
                            
                            JToken queueMessageValue2 = actionValue["queueMessage"];
                            if (queueMessageValue2 != null)
                            {
                                JobQueueMessage queueMessageInstance2 = new JobQueueMessage();
                                actionInstance.QueueMessage = queueMessageInstance2;
                                
                                JToken storageAccountValue2 = queueMessageValue2["storageAccount"];
                                if (storageAccountValue2 != null)
                                {
                                    string storageAccountInstance2 = (string)storageAccountValue2;
                                    queueMessageInstance2.StorageAccountName = storageAccountInstance2;
                                }
                                
                                JToken queueNameValue2 = queueMessageValue2["queueName"];
                                if (queueNameValue2 != null)
                                {
                                    string queueNameInstance2 = (string)queueNameValue2;
                                    queueMessageInstance2.QueueName = queueNameInstance2;
                                }
                                
                                JToken sasTokenValue2 = queueMessageValue2["sasToken"];
                                if (sasTokenValue2 != null)
                                {
                                    string sasTokenInstance2 = (string)sasTokenValue2;
                                    queueMessageInstance2.SasToken = sasTokenInstance2;
                                }
                                
                                JToken messageValue2 = queueMessageValue2["message"];
                                if (messageValue2 != null)
                                {
                                    string messageInstance2 = (string)messageValue2;
                                    queueMessageInstance2.Message = messageInstance2;
                                }
                            }
                        }
                        
                        JToken recurrenceValue = responseDoc["recurrence"];
                        if (recurrenceValue != null)
                        {
                            JobRecurrence recurrenceInstance = new JobRecurrence();
                            jobInstance.Recurrence = recurrenceInstance;
                            
                            JToken frequencyValue = recurrenceValue["frequency"];
                            if (frequencyValue != null)
                            {
                                JobRecurrenceFrequency frequencyInstance = SchedulerClient.ParseJobRecurrenceFrequency((string)frequencyValue);
                                recurrenceInstance.Frequency = frequencyInstance;
                            }
                            
                            JToken intervalValue = recurrenceValue["interval"];
                            if (intervalValue != null)
                            {
                                int intervalInstance = (int)intervalValue;
                                recurrenceInstance.Interval = intervalInstance;
                            }
                            
                            JToken countValue = recurrenceValue["count"];
                            if (countValue != null)
                            {
                                int countInstance = (int)countValue;
                                recurrenceInstance.Count = countInstance;
                            }
                            
                            JToken endTimeValue = recurrenceValue["endTime"];
                            if (endTimeValue != null)
                            {
                                DateTime endTimeInstance = (DateTime)endTimeValue;
                                recurrenceInstance.EndTime = endTimeInstance;
                            }
                            
                            JToken scheduleValue = recurrenceValue["schedule"];
                            if (scheduleValue != null)
                            {
                                JobRecurrenceSchedule scheduleInstance = new JobRecurrenceSchedule();
                                recurrenceInstance.Schedule = scheduleInstance;
                                
                                JArray minutesArray = (JArray)scheduleValue["minutes"];
                                if (minutesArray != null)
                                {
                                    foreach (JToken minutesValue in minutesArray)
                                    {
                                        scheduleInstance.Minutes.Add((int)minutesValue);
                                    }
                                }
                                
                                JArray hoursArray = (JArray)scheduleValue["hours"];
                                if (hoursArray != null)
                                {
                                    foreach (JToken hoursValue in hoursArray)
                                    {
                                        scheduleInstance.Hours.Add((int)hoursValue);
                                    }
                                }
                                
                                JArray daysArray = (JArray)scheduleValue["days"];
                                if (daysArray != null)
                                {
                                    foreach (JToken daysValue in daysArray)
                                    {
                                        scheduleInstance.Days.Add(SchedulerClient.ParseJobScheduleDay((string)daysValue));
                                    }
                                }
                                
                                JArray monthsArray = (JArray)scheduleValue["months"];
                                if (monthsArray != null)
                                {
                                    foreach (JToken monthsValue in monthsArray)
                                    {
                                        scheduleInstance.Months.Add((int)monthsValue);
                                    }
                                }
                                
                                JArray monthDaysArray = (JArray)scheduleValue["monthDays"];
                                if (monthDaysArray != null)
                                {
                                    foreach (JToken monthDaysValue in monthDaysArray)
                                    {
                                        scheduleInstance.MonthDays.Add((int)monthDaysValue);
                                    }
                                }
                                
                                JArray monthlyOccurrencesArray = (JArray)scheduleValue["monthlyOccurrences"];
                                if (monthlyOccurrencesArray != null)
                                {
                                    foreach (JToken monthlyOccurrencesValue in monthlyOccurrencesArray)
                                    {
                                        JobScheduleMonthlyOccurrence jobScheduleMonthlyOccurrenceInstance = new JobScheduleMonthlyOccurrence();
                                        scheduleInstance.MonthlyOccurrences.Add(jobScheduleMonthlyOccurrenceInstance);
                                        
                                        JToken dayValue = monthlyOccurrencesValue["day"];
                                        if (dayValue != null)
                                        {
                                            JobScheduleDay dayInstance = SchedulerClient.ParseJobScheduleDay((string)dayValue);
                                            jobScheduleMonthlyOccurrenceInstance.Day = dayInstance;
                                        }
                                        
                                        JToken occurrenceValue = monthlyOccurrencesValue["occurrence"];
                                        if (occurrenceValue != null)
                                        {
                                            int occurrenceInstance = (int)occurrenceValue;
                                            jobScheduleMonthlyOccurrenceInstance.Occurrence = occurrenceInstance;
                                        }
                                    }
                                }
                            }
                        }
                        
                        JToken statusValue = responseDoc["status"];
                        if (statusValue != null)
                        {
                            JobStatus statusInstance = new JobStatus();
                            jobInstance.Status = statusInstance;
                            
                            JToken lastExecutionTimeValue = statusValue["lastExecutionTime"];
                            if (lastExecutionTimeValue != null)
                            {
                                DateTime lastExecutionTimeInstance = (DateTime)lastExecutionTimeValue;
                                statusInstance.LastExecutionTime = lastExecutionTimeInstance;
                            }
                            
                            JToken nextExecutionTimeValue = statusValue["nextExecutionTime"];
                            if (nextExecutionTimeValue != null)
                            {
                                DateTime nextExecutionTimeInstance = (DateTime)nextExecutionTimeValue;
                                statusInstance.NextExecutionTime = nextExecutionTimeInstance;
                            }
                            
                            JToken executionCountValue = statusValue["executionCount"];
                            if (executionCountValue != null)
                            {
                                int executionCountInstance = (int)executionCountValue;
                                statusInstance.ExecutionCount = executionCountInstance;
                            }
                            
                            JToken failureCountValue = statusValue["failureCount"];
                            if (failureCountValue != null)
                            {
                                int failureCountInstance = (int)failureCountValue;
                                statusInstance.FailureCount = failureCountInstance;
                            }
                            
                            JToken faultedCountValue = statusValue["faultedCount"];
                            if (faultedCountValue != null)
                            {
                                int faultedCountInstance = (int)faultedCountValue;
                                statusInstance.FaultedCount = faultedCountInstance;
                            }
                        }
                        
                        JToken stateValue2 = responseDoc["state"];
                        if (stateValue2 != null)
                        {
                            JobState stateInstance = SchedulerClient.ParseJobState((string)stateValue2);
                            jobInstance.State = stateInstance;
                        }
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
